@@ -351,6 +351,8 @@ print_modem_info (void)
     mmcli_output_string           (MMC_F_HARDWARE_MANUFACTURER,           mm_modem_get_manufacturer (ctx->modem));
     mmcli_output_string           (MMC_F_HARDWARE_MODEL,                  mm_modem_get_model (ctx->modem));
     mmcli_output_string           (MMC_F_HARDWARE_REVISION,               mm_modem_get_revision (ctx->modem));
+    mmcli_output_string           (MMC_F_HARDWARE_CARRIER_CONF,           mm_modem_get_carrier_configuration (ctx->modem));
+    mmcli_output_string           (MMC_F_HARDWARE_CARRIER_CONF_REV,       mm_modem_get_carrier_configuration_revision (ctx->modem));
     mmcli_output_string           (MMC_F_HARDWARE_HW_REVISION,            mm_modem_get_hardware_revision (ctx->modem));
     mmcli_output_string_multiline (MMC_F_HARDWARE_SUPPORTED_CAPABILITIES, supported_capabilities_string);
     mmcli_output_string_multiline (MMC_F_HARDWARE_CURRENT_CAPABILITIES,   current_capabilities_string);
@@ -432,7 +434,7 @@ print_modem_info (void)
         mmcli_output_pco_list    (pco_list);
 
         g_free (facility_locks);
-        mm_pco_list_free (pco_list);
+        g_list_free_full (pco_list, g_object_unref);
     }
 
     /* CDMA */
@@ -900,6 +902,19 @@ state_changed (MMModem                  *modem,
     fflush (stdout);
 }
 
+static void
+device_removed (MMManager *manager,
+                MMObject  *object)
+{
+    if (object != ctx->object)
+        return;
+
+    g_print ("\t%s: Removed\n", mm_object_get_path (object));
+    fflush (stdout);
+
+    mmcli_async_operation_done ();
+}
+
 static void
 get_modem_ready (GObject      *source,
                  GAsyncResult *result,
@@ -930,6 +945,11 @@ get_modem_ready (GObject      *source,
                           G_CALLBACK (state_changed),
                           NULL);
 
+        g_signal_connect (ctx->manager,
+                          "object-removed",
+                          G_CALLBACK (device_removed),
+                          NULL);
+
         current = mm_modem_get_state (ctx->modem);
         g_print ("\t%s: Initial state, '%s'\n",
                  mm_object_get_path (ctx->object),
