@@ -58,6 +58,8 @@ struct _MMLocation3gppPrivate {
  * Gets the Mobile Country Code of the 3GPP network.
  *
  * Returns: the MCC, or 0 if unknown.
+ *
+ * Since: 1.0
  */
 guint
 mm_location_3gpp_get_mobile_country_code (MMLocation3gpp *self)
@@ -67,6 +69,9 @@ mm_location_3gpp_get_mobile_country_code (MMLocation3gpp *self)
     return self->priv->mobile_country_code;
 }
 
+/**
+ * mm_location_3gpp_set_mobile_country_code: (skip)
+ */
 gboolean
 mm_location_3gpp_set_mobile_country_code (MMLocation3gpp *self,
                                           guint mobile_country_code)
@@ -94,6 +99,8 @@ mm_location_3gpp_set_mobile_country_code (MMLocation3gpp *self,
  * be 0.
  *
  * Returns: the MNC, or 0 if unknown.
+ *
+ * Since: 1.0
  */
 guint
 mm_location_3gpp_get_mobile_network_code (MMLocation3gpp *self)
@@ -103,6 +110,9 @@ mm_location_3gpp_get_mobile_network_code (MMLocation3gpp *self)
     return self->priv->mobile_network_code;
 }
 
+/**
+ * mm_location_3gpp_set_mobile_network_code: (skip)
+ */
 gboolean
 mm_location_3gpp_set_mobile_network_code (MMLocation3gpp *self,
                                           guint mobile_network_code)
@@ -127,6 +137,8 @@ mm_location_3gpp_set_mobile_network_code (MMLocation3gpp *self,
  * Gets the location area code of the 3GPP network.
  *
  * Returns: the location area code, or 0 if unknown.
+ *
+ * Since: 1.0
  */
 gulong
 mm_location_3gpp_get_location_area_code (MMLocation3gpp *self)
@@ -136,6 +148,9 @@ mm_location_3gpp_get_location_area_code (MMLocation3gpp *self)
     return self->priv->location_area_code;
 }
 
+/**
+ * mm_location_3gpp_set_location_area_code: (skip)
+ */
 gboolean
 mm_location_3gpp_set_location_area_code (MMLocation3gpp *self,
                                          gulong location_area_code)
@@ -159,6 +174,8 @@ mm_location_3gpp_set_location_area_code (MMLocation3gpp *self,
  * Gets the cell ID of the 3GPP network.
  *
  * Returns: the cell ID, or 0 if unknown.
+ *
+ * Since: 1.0
  */
 gulong
 mm_location_3gpp_get_cell_id (MMLocation3gpp *self)
@@ -168,6 +185,9 @@ mm_location_3gpp_get_cell_id (MMLocation3gpp *self)
     return self->priv->cell_id;
 }
 
+/**
+ * mm_location_3gpp_set_cell_id: (skip)
+ */
 gboolean
 mm_location_3gpp_set_cell_id (MMLocation3gpp *self,
                               gulong cell_id)
@@ -191,6 +211,8 @@ mm_location_3gpp_set_cell_id (MMLocation3gpp *self,
  * Gets the location area code of the 3GPP network.
  *
  * Returns: the location area code, or 0 if unknown.
+ *
+ * Since: 1.10
  */
 gulong
 mm_location_3gpp_get_tracking_area_code (MMLocation3gpp *self)
@@ -200,6 +222,9 @@ mm_location_3gpp_get_tracking_area_code (MMLocation3gpp *self)
     return self->priv->tracking_area_code;
 }
 
+/**
+ * mm_location_3gpp_set_tracking_area_code: (skip)
+ */
 gboolean
 mm_location_3gpp_set_tracking_area_code (MMLocation3gpp *self,
                                          gulong tracking_area_code)
@@ -216,6 +241,9 @@ mm_location_3gpp_set_tracking_area_code (MMLocation3gpp *self,
 
 /*****************************************************************************/
 
+/**
+ * mm_location_3gpp_reset: (skip)
+ */
 gboolean
 mm_location_3gpp_reset (MMLocation3gpp *self)
 {
@@ -240,6 +268,9 @@ mm_location_3gpp_reset (MMLocation3gpp *self)
 
 /*****************************************************************************/
 
+/**
+ * mm_location_3gpp_get_string_variant: (skip)
+ */
 GVariant *
 mm_location_3gpp_get_string_variant (MMLocation3gpp *self)
 {
@@ -260,7 +291,7 @@ mm_location_3gpp_get_string_variant (MMLocation3gpp *self)
                                self->priv->cell_id,
                                self->priv->tracking_area_code);
 
-        variant = g_variant_new_string (str);
+        variant = g_variant_ref_sink (g_variant_new_string (str));
         g_free (str);
     }
 
@@ -325,6 +356,9 @@ validate_numeric_string_content (const gchar *display,
     return TRUE;
 }
 
+/**
+ * mm_location_3gpp_new_from_string_variant: (skip)
+ */
 MMLocation3gpp *
 mm_location_3gpp_new_from_string_variant (GVariant *string,
                                           GError **error)
@@ -377,6 +411,9 @@ mm_location_3gpp_new_from_string_variant (GVariant *string,
 
 /*****************************************************************************/
 
+/**
+ * mm_location_3gpp_new: (skip)
+ */
 MMLocation3gpp *
 mm_location_3gpp_new (void)
 {
