@@ -10,9 +10,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details:
  *
- * Copyright (C) 2016 Trimble Navigation Limited
  * Copyright (C) 2014 Aleksander Morgado <aleksander@aleksander.es>
- * Contributor: Matthew Stanger <matthew_stanger@trimble.com>
+ * Copyright (C) 2016 Trimble Navigation Limited
+ * Copyright (C) 2016 Matthew Stanger <matthew_stanger@trimble.com>
+ * Copyright (C) 2019 Purism SPC
  */
 
 #include <config.h>
@@ -201,7 +202,7 @@ mm_cinterion_parse_scfg_response (const gchar *response,
     r = g_regex_new ("\\^SCFG:\\s*\"Radio/Band\",\\s*\"?([0-9a-fA-F]*)\"?", 0, 0, NULL);
     g_assert (r != NULL);
 
-    if (g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, NULL)) {
+    if (g_regex_match (r, response, 0, &match_info)) {
         gchar *currentstr;
         guint current = 0;
 
@@ -441,7 +442,7 @@ mm_cinterion_parse_sind_response (const gchar *response,
     r = g_regex_new ("\\^SIND:\\s*(.*),(\\d+),(\\d+)(\\r\\n)?", 0, 0, NULL);
     g_assert (r != NULL);
 
-    if (g_regex_match_full (r, response, strlen (response), 0, 0, &match_info, NULL)) {
+    if (g_regex_match (r, response, 0, &match_info)) {
         if (description) {
             *description = mm_get_string_unquoted_from_match_info (match_info, 1);
             if (*description == NULL)
@@ -667,3 +668,201 @@ mm_cinterion_get_access_technology_from_sind_psinfo (guint val)
         return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
     }
 }
+
+/*****************************************************************************/
+/* ^SLCC psinfo helper */
+
+GRegex *
+mm_cinterion_get_slcc_regex (void)
+{
+    /* The list of active calls displayed with this URC will always be terminated
+     * with an empty line preceded by prefix "^SLCC: ", in order to indicate the end
+     * of the list.
+     */
+    return g_regex_new ("\\r\\n(\\^SLCC: .*\\r\\n)*\\^SLCC: \\r\\n",
+                        G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+}
+
+static void
+cinterion_call_info_free (MMCallInfo *info)
+{
+    if (!info)
+        return;
+    g_free (info->number);
+    g_slice_free (MMCallInfo, info);
+}
+
+gboolean
+mm_cinterion_parse_slcc_list (const gchar *str,
+                              GList      **out_list,
+                              GError     **error)
+{
+    GRegex     *r;
+    GList      *list = NULL;
+    GError     *inner_error = NULL;
+    GMatchInfo *match_info  = NULL;
+
+    static const MMCallDirection cinterion_call_direction[] = {
+        [0] = MM_CALL_DIRECTION_OUTGOING,
+        [1] = MM_CALL_DIRECTION_INCOMING,
+    };
+
+    static const MMCallState cinterion_call_state[] = {
+        [0] = MM_CALL_STATE_ACTIVE,
+        [1] = MM_CALL_STATE_HELD,
+        [2] = MM_CALL_STATE_DIALING,     /* Dialing  (MOC) */
+        [3] = MM_CALL_STATE_RINGING_OUT, /* Alerting (MOC) */
+        [4] = MM_CALL_STATE_RINGING_IN,  /* Incoming (MTC) */
+        [5] = MM_CALL_STATE_WAITING,     /* Waiting  (MTC) */
+    };
+
+    g_assert (out_list);
+
+    /*
+     *         1      2      3       4       5       6            7         8     9
+     *  ^SLCC: <idx>, <dir>, <stat>, <mode>, <mpty>, <Reserved>[, <number>, <type>[,<alpha>]]
+     *  [^SLCC: <idx>, <dir>, <stat>, <mode>, <mpty>, <Reserved>[, <number>, <type>[,<alpha>]]]
+     *  [... ]
+     *  ^SLCC :
+     */
+
+    r = g_regex_new ("\\^SLCC:\\s*(\\d+),\\s*(\\d+),\\s*(\\d+),\\s*(\\d+),\\s*(\\d+),\\s*(\\d+)" /* mandatory fields */
+                     "(?:,\\s*([^,]*),\\s*(\\d+)"                                                /* number and type */
+                     "(?:,\\s*([^,]*)"                                                           /* alpha */
+                     ")?)?$",
+                     G_REGEX_RAW | G_REGEX_MULTILINE | G_REGEX_NEWLINE_CRLF,
+                     G_REGEX_MATCH_NEWLINE_CRLF,
+                     NULL);
+    g_assert (r != NULL);
+
+    g_regex_match_full (r, str, strlen (str), 0, 0, &match_info, &inner_error);
+    if (inner_error)
+        goto out;
+
+    /* Parse the results */
+    while (g_match_info_matches (match_info)) {
+        MMCallInfo *call_info;
+        guint       aux;
+
+        call_info = g_slice_new0 (MMCallInfo);
+
+        if (!mm_get_uint_from_match_info (match_info, 1, &call_info->index)) {
+            mm_warn ("couldn't parse call index from ^SLCC line");
+            goto next;
+        }
+
+        if (!mm_get_uint_from_match_info (match_info, 2, &aux) ||
+            (aux >= G_N_ELEMENTS (cinterion_call_direction))) {
+            mm_warn ("couldn't parse call direction from ^SLCC line");
+            goto next;
+        }
+        call_info->direction = cinterion_call_direction[aux];
+
+        if (!mm_get_uint_from_match_info (match_info, 3, &aux) ||
+            (aux >= G_N_ELEMENTS (cinterion_call_state))) {
+            mm_warn ("couldn't parse call state from ^SLCC line");
+            goto next;
+        }
+        call_info->state = cinterion_call_state[aux];
+
+        if (g_match_info_get_match_count (match_info) >= 8)
+            call_info->number = mm_get_string_unquoted_from_match_info (match_info, 7);
+
+        list = g_list_append (list, call_info);
+        call_info = NULL;
+
+    next:
+        cinterion_call_info_free (call_info);
+        g_match_info_next (match_info, NULL);
+    }
+
+out:
+    g_clear_pointer (&match_info, g_match_info_free);
+    g_regex_unref (r);
+
+    if (inner_error) {
+        mm_cinterion_call_info_list_free (list);
+        g_propagate_error (error, inner_error);
+        return FALSE;
+    }
+
+    *out_list = list;
+
+    return TRUE;
+}
+
+void
+mm_cinterion_call_info_list_free (GList *call_info_list)
+{
+    g_list_free_full (call_info_list, (GDestroyNotify) cinterion_call_info_free);
+}
+
+/*****************************************************************************/
+/* +CTZU URC helpers */
+
+GRegex *
+mm_cinterion_get_ctzu_regex (void)
+{
+    /*
+     * From PLS-8 AT command spec:
+     *  +CTZU:<nitzUT>, <nitzTZ>[, <nitzDST>]
+     * E.g.:
+     *  +CTZU: "19/07/09,10:19:15",+08,1
+     */
+
+    return g_regex_new ("\\r\\n\\+CTZU:\\s*\"(\\d+)\\/(\\d+)\\/(\\d+),(\\d+):(\\d+):(\\d+)\",([\\-\\+\\d]+)(?:,(\\d+))?(?:\\r\\n)?",
+                        G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+}
+
+gboolean
+mm_cinterion_parse_ctzu_urc (GMatchInfo         *match_info,
+                             gchar             **iso8601p,
+                             MMNetworkTimezone **tzp,
+                             GError            **error)
+{
+    guint year = 0, month = 0, day = 0, hour = 0, minute = 0, second = 0, dst = 0;
+    gint tz = 0;
+
+    if (!mm_get_uint_from_match_info (match_info, 1, &year)   ||
+        !mm_get_uint_from_match_info (match_info, 2, &month)  ||
+        !mm_get_uint_from_match_info (match_info, 3, &day)    ||
+        !mm_get_uint_from_match_info (match_info, 4, &hour)   ||
+        !mm_get_uint_from_match_info (match_info, 5, &minute) ||
+        !mm_get_uint_from_match_info (match_info, 6, &second) ||
+        !mm_get_int_from_match_info  (match_info, 7, &tz)) {
+        g_set_error_literal (error,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_FAILED,
+                             "Failed to parse +CTZU URC");
+        return FALSE;
+    }
+
+    /* adjust year */
+    if (year < 100)
+        year += 2000;
+
+    /*
+     * tz = timezone offset in 15 minute intervals
+     */
+    if (iso8601p) {
+        /* Return ISO-8601 format date/time string */
+        *iso8601p = mm_new_iso8601_time (year, month, day, hour,
+                                         minute, second,
+                                         TRUE, tz * 15);
+    }
+
+    if (tzp) {
+        *tzp = mm_network_timezone_new ();
+        mm_network_timezone_set_offset (*tzp, tz * 15);
+    }
+
+    /* dst flag is optional in the URC
+    *
+     * tz = timezone offset in 15 minute intervals
+     * dst = daylight adjustment, 0 = none, 1 = 1 hour, 2 = 2 hours
+     */
+    if (tzp && mm_get_uint_from_match_info (match_info, 8, &dst))
+        mm_network_timezone_set_dst_offset (*tzp, dst * 60);
+
+    return TRUE;
+}
