@@ -667,6 +667,195 @@ test_smong_response_other (void)
     common_test_smong_response (response, MM_MODEM_ACCESS_TECHNOLOGY_GPRS);
 }
 
+/*****************************************************************************/
+/* Test ^SLCC URCs */
+
+static void
+common_test_slcc_urc (const gchar               *urc,
+                      const MMCallInfo *expected_call_info_list,
+                      guint                      expected_call_info_list_size)
+{
+    GError     *error = NULL;
+    GRegex     *slcc_regex = NULL;
+    gboolean    result;
+    GMatchInfo *match_info = NULL;
+    gchar      *str;
+    GList      *call_info_list = NULL;
+    GList      *l;
+
+
+    slcc_regex = mm_cinterion_get_slcc_regex ();
+
+    /* Same matching logic as done in MMSerialPortAt when processing URCs! */
+    result = g_regex_match_full (slcc_regex, urc, -1, 0, 0, &match_info, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+
+    /* read full matched content */
+    str = g_match_info_fetch (match_info, 0);
+    g_assert (str);
+
+    result = mm_cinterion_parse_slcc_list (str, &call_info_list, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+
+    g_debug ("found %u calls", g_list_length (call_info_list));
+
+    if (expected_call_info_list) {
+        g_assert (call_info_list);
+        g_assert_cmpuint (g_list_length (call_info_list), ==, expected_call_info_list_size);
+    } else
+        g_assert (!call_info_list);
+
+    for (l = call_info_list; l; l = g_list_next (l)) {
+        const MMCallInfo *call_info = (const MMCallInfo *)(l->data);
+        gboolean                   found = FALSE;
+        guint                      i;
+
+        g_debug ("call at index %u: direction %s, state %s, number %s",
+                 call_info->index,
+                 mm_call_direction_get_string (call_info->direction),
+                 mm_call_state_get_string (call_info->state),
+                 call_info->number ? call_info->number : "n/a");
+
+        for (i = 0; !found && i < expected_call_info_list_size; i++)
+            found = ((call_info->index == expected_call_info_list[i].index) &&
+                     (call_info->direction  == expected_call_info_list[i].direction) &&
+                     (call_info->state  == expected_call_info_list[i].state) &&
+                     (g_strcmp0 (call_info->number, expected_call_info_list[i].number) == 0));
+
+        g_assert (found);
+    }
+
+    g_match_info_free (match_info);
+    g_regex_unref (slcc_regex);
+    g_free (str);
+
+    mm_cinterion_call_info_list_free (call_info_list);
+}
+
+static void
+test_slcc_urc_empty (void)
+{
+    const gchar *urc = "\r\n^SLCC: \r\n";
+
+    common_test_slcc_urc (urc, NULL, 0);
+}
+
+static void
+test_slcc_urc_single (void)
+{
+    static const MMCallInfo expected_call_info_list[] = {
+        { 1, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_ACTIVE, "123456789" }
+    };
+
+    const gchar *urc =
+        "\r\n^SLCC: 1,1,0,0,0,0,\"123456789\",161"
+        "\r\n^SLCC: \r\n";
+
+    common_test_slcc_urc (urc, expected_call_info_list, G_N_ELEMENTS (expected_call_info_list));
+}
+
+static void
+test_slcc_urc_multiple (void)
+{
+    static const MMCallInfo expected_call_info_list[] = {
+        { 1, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_ACTIVE,  NULL        },
+        { 2, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_ACTIVE,  "123456789" },
+        { 3, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_ACTIVE,  "987654321" },
+    };
+
+    const gchar *urc =
+        "\r\n^SLCC: 1,1,0,0,1,0" /* number unknown */
+        "\r\n^SLCC: 2,1,0,0,1,0,\"123456789\",161"
+        "\r\n^SLCC: 3,1,0,0,1,0,\"987654321\",161,\"Alice\""
+        "\r\n^SLCC: \r\n";
+
+    common_test_slcc_urc (urc, expected_call_info_list, G_N_ELEMENTS (expected_call_info_list));
+}
+
+static void
+test_slcc_urc_complex (void)
+{
+    static const MMCallInfo expected_call_info_list[] = {
+        { 1, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_ACTIVE,  "123456789" },
+        { 2, MM_CALL_DIRECTION_INCOMING, MM_CALL_STATE_WAITING, "987654321" },
+    };
+
+    const gchar *urc =
+        "\r\n^CIEV: 1,0" /* some different URC before our match */
+        "\r\n^SLCC: 1,1,0,0,0,0,\"123456789\",161"
+        "\r\n^SLCC: 2,1,5,0,0,0,\"987654321\",161"
+        "\r\n^SLCC: \r\n"
+        "\r\n^CIEV: 1,0" /* some different URC after our match */;
+
+    common_test_slcc_urc (urc, expected_call_info_list, G_N_ELEMENTS (expected_call_info_list));
+}
+
+/*****************************************************************************/
+/* Test +CTZU URCs */
+
+static void
+common_test_ctzu_urc (const gchar *urc,
+                      const gchar *expected_iso8601,
+                      gint         expected_offset,
+                      gint         expected_dst_offset)
+{
+    GError            *error = NULL;
+    GRegex            *ctzu_regex = NULL;
+    gboolean           result;
+    GMatchInfo        *match_info = NULL;
+    gchar             *iso8601;
+    MMNetworkTimezone *tz = NULL;
+
+    ctzu_regex = mm_cinterion_get_ctzu_regex ();
+
+    /* Same matching logic as done in MMSerialPortAt when processing URCs! */
+    result = g_regex_match_full (ctzu_regex, urc, -1, 0, 0, &match_info, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+
+    result = mm_cinterion_parse_ctzu_urc (match_info, &iso8601, &tz, &error);
+    g_assert_no_error (error);
+    g_assert (result);
+
+    g_assert (iso8601);
+    g_assert_cmpstr (expected_iso8601, ==, iso8601);
+    g_free (iso8601);
+
+    g_assert (tz);
+    g_assert_cmpint (expected_offset, ==, mm_network_timezone_get_offset (tz));
+
+    if (expected_dst_offset >= 0)
+        g_assert_cmpuint ((guint)expected_dst_offset, ==, mm_network_timezone_get_dst_offset (tz));
+
+    g_object_unref (tz);
+    g_match_info_free (match_info);
+    g_regex_unref (ctzu_regex);
+}
+
+static void
+test_ctzu_urc_simple (void)
+{
+    const gchar *urc = "\r\n+CTZU: \"19/07/09,11:15:40\",+08\r\n";
+    const gchar *expected_iso8601    = "2019-07-09T11:15:40+02:00";
+    gint         expected_offset     = 120;
+    gint         expected_dst_offset = -1; /* not given */
+
+    common_test_ctzu_urc (urc, expected_iso8601, expected_offset, expected_dst_offset);
+}
+
+static void
+test_ctzu_urc_full (void)
+{
+    const gchar *urc = "\r\n+CTZU: \"19/07/09,11:15:40\",+08,1\r\n";
+    const gchar *expected_iso8601    = "2019-07-09T11:15:40+02:00";
+    gint         expected_offset     = 120;
+    gint         expected_dst_offset = 60;
+
+    common_test_ctzu_urc (urc, expected_iso8601, expected_offset, expected_dst_offset);
+}
+
 /*****************************************************************************/
 
 void
@@ -676,17 +865,17 @@ _mm_log (const char *loc,
          const char *fmt,
          ...)
 {
-#if defined ENABLE_TEST_MESSAGE_TRACES
-    /* Dummy log function */
     va_list args;
     gchar *msg;
 
+    if (!g_test_verbose ())
+        return;
+
     va_start (args, fmt);
     msg = g_strdup_vprintf (fmt, args);
     va_end (args);
     g_print ("%s\n", msg);
     g_free (msg);
-#endif
 }
 
 int main (int argc, char **argv)
@@ -706,6 +895,12 @@ int main (int argc, char **argv)
     g_test_add_func ("/MM/cinterion/sind/response/simstatus", test_sind_response_simstatus);
     g_test_add_func ("/MM/cinterion/smong/response/tc63i",    test_smong_response_tc63i);
     g_test_add_func ("/MM/cinterion/smong/response/other",    test_smong_response_other);
+    g_test_add_func ("/MM/cinterion/slcc/urc/empty",          test_slcc_urc_empty);
+    g_test_add_func ("/MM/cinterion/slcc/urc/single",         test_slcc_urc_single);
+    g_test_add_func ("/MM/cinterion/slcc/urc/multiple",       test_slcc_urc_multiple);
+    g_test_add_func ("/MM/cinterion/slcc/urc/complex",        test_slcc_urc_complex);
+    g_test_add_func ("/MM/cinterion/ctzu/urc/simple",         test_ctzu_urc_simple);
+    g_test_add_func ("/MM/cinterion/ctzu/urc/full",           test_ctzu_urc_full);
 
     return g_test_run ();
 }
