@@ -35,23 +35,14 @@ MM_PLUGIN_DEFINE_MINOR_VERSION
 /* Custom init */
 
 typedef struct {
-    MMPortProbe *probe;
     MMPortSerialAt *port;
-    GCancellable *cancellable;
-    GSimpleAsyncResult *result;
     guint retries;
 } LongcheerCustomInitContext;
 
 static void
-longcheer_custom_init_context_complete_and_free (LongcheerCustomInitContext *ctx)
+longcheer_custom_init_context_free (LongcheerCustomInitContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-
-    if (ctx->cancellable)
-        g_object_unref (ctx->cancellable);
     g_object_unref (ctx->port);
-    g_object_unref (ctx->probe);
-    g_object_unref (ctx->result);
     g_slice_free (LongcheerCustomInitContext, ctx);
 }
 
@@ -60,15 +51,15 @@ longcheer_custom_init_finish (MMPortProbe *probe,
                               GAsyncResult *result,
                               GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error);
+    return g_task_propagate_boolean (G_TASK (result), error);
 }
 
-static void longcheer_custom_init_step (LongcheerCustomInitContext *ctx);
+static void longcheer_custom_init_step (GTask *task);
 
 static void
 gmr_ready (MMPortSerialAt *port,
            GAsyncResult *res,
-           LongcheerCustomInitContext *ctx)
+           GTask *task)
 {
     const gchar *p;
     const gchar *response;
@@ -76,9 +67,11 @@ gmr_ready (MMPortSerialAt *port,
 
     response = mm_port_serial_at_command_finish (port, res, &error);
     if (error) {
+        g_error_free (error);
+
         /* Just retry... */
-        longcheer_custom_init_step (ctx);
-        goto out;
+        longcheer_custom_init_step (task);
+        return;
     }
 
     /* Note the lack of a ':' on the GMR; the X200 doesn't send one */
@@ -90,42 +83,43 @@ gmr_ready (MMPortSerialAt *port,
          * does not support since it uses a different chipset even though the
          * X060s and the X200 have the exact same USB VID and PID.
          */
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_UNSUPPORTED,
-                                         "X200 cannot be supported with the Longcheer plugin");
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_UNSUPPORTED,
+                                 "X200 cannot be supported with the Longcheer plugin");
     } else {
         mm_dbg ("(Longcheer) device is not a X200");
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        g_task_return_boolean (task, TRUE);
     }
-
-    longcheer_custom_init_context_complete_and_free (ctx);
-
-out:
-    if (error)
-        g_error_free (error);
+    g_object_unref (task);
 }
 
 static void
-longcheer_custom_init_step (LongcheerCustomInitContext *ctx)
+longcheer_custom_init_step (GTask *task)
 {
+    LongcheerCustomInitContext *ctx;
+    GCancellable *cancellable;
+
+    ctx = g_task_get_task_data (task);
+    cancellable = g_task_get_cancellable (task);
+
     /* If cancelled, end */
-    if (g_cancellable_is_cancelled (ctx->cancellable)) {
+    if (g_cancellable_is_cancelled (cancellable)) {
         mm_dbg ("(Longcheer) no need to keep on running custom init in (%s)",
                 mm_port_get_device (MM_PORT (ctx->port)));
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        longcheer_custom_init_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
     if (ctx->retries == 0) {
         /* In this case, we need the AT command result to decide whether we can
          * support this modem or not, so really fail if we didn't get it. */
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Couldn't get device revision information");
-        longcheer_custom_init_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Couldn't get device revision information");
+        g_object_unref (task);
         return;
     }
 
@@ -136,9 +130,9 @@ longcheer_custom_init_step (LongcheerCustomInitContext *ctx)
         3,
         FALSE, /* raw */
         FALSE, /* allow_cached */
-        ctx->cancellable,
+        cancellable,
         (GAsyncReadyCallback)gmr_ready,
-        ctx);
+        task);
 }
 
 static void
@@ -150,17 +144,19 @@ longcheer_custom_init (MMPortProbe *probe,
 {
     MMDevice *device;
     LongcheerCustomInitContext *ctx;
+    GTask *task;
 
     ctx = g_slice_new (LongcheerCustomInitContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (probe),
-                                             callback,
-                                             user_data,
-                                             longcheer_custom_init);
-    ctx->probe = g_object_ref (probe);
     ctx->port = g_object_ref (port);
-    ctx->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
     ctx->retries = 3;
 
+    task = g_task_new (probe, cancellable, callback, user_data);
+    /* Clears the check-cancellable flag of the task as we expect the task to
+     * return TRUE upon cancellation.
+     */
+    g_task_set_check_cancellable (task, FALSE);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)longcheer_custom_init_context_free);
+
     /* TCT/Alcatel in their infinite wisdom assigned the same USB VID/PID to
      * the x060s (Longcheer firmware) and the x200 (something else) and thus
      * we can't tell them apart via udev rules.  Worse, they both report the
@@ -174,76 +170,32 @@ longcheer_custom_init (MMPortProbe *probe,
     if (mm_device_get_vendor (device) != 0x1bbb ||
         mm_device_get_product (device) != 0x0000) {
         /* If not exactly this vendor/product, just skip */
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        longcheer_custom_init_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
-    longcheer_custom_init_step (ctx);
+    longcheer_custom_init_step (task);
 }
 
 /*****************************************************************************/
 
 static MMBaseModem *
 create_modem (MMPlugin *self,
-              const gchar *sysfs_path,
+              const gchar *uid,
               const gchar **drivers,
               guint16 vendor,
               guint16 product,
               GList *probes,
               GError **error)
 {
-    return MM_BASE_MODEM (mm_broadband_modem_longcheer_new (sysfs_path,
+    return MM_BASE_MODEM (mm_broadband_modem_longcheer_new (uid,
                                                             drivers,
                                                             mm_plugin_get_name (self),
                                                             vendor,
                                                             product));
 }
 
-static gboolean
-grab_port (MMPlugin *self,
-           MMBaseModem *modem,
-           MMPortProbe *probe,
-           GError **error)
-{
-    GUdevDevice *port;
-    MMPortType ptype;
-    MMPortSerialAtFlag pflags = MM_PORT_SERIAL_AT_FLAG_NONE;
-
-    port = mm_port_probe_peek_port (probe);
-    ptype = mm_port_probe_get_port_type (probe);
-
-    /* Look for port type hints; just probing can't distinguish which port should
-     * be the data/primary port on these devices.  We have to tag them based on
-     * what the Windows .INF files say the port layout should be.
-     */
-    if (g_udev_device_get_property_as_boolean (port, "ID_MM_LONGCHEER_PORT_TYPE_MODEM")) {
-        mm_dbg ("longcheer: AT port '%s/%s' flagged as primary",
-                mm_port_probe_get_port_subsys (probe),
-                mm_port_probe_get_port_name (probe));
-        pflags = MM_PORT_SERIAL_AT_FLAG_PRIMARY;
-    } else if (g_udev_device_get_property_as_boolean (port, "ID_MM_LONGCHEER_PORT_TYPE_AUX")) {
-        mm_dbg ("longcheer: AT port '%s/%s' flagged as secondary",
-                mm_port_probe_get_port_subsys (probe),
-                mm_port_probe_get_port_name (probe));
-        pflags = MM_PORT_SERIAL_AT_FLAG_SECONDARY;
-    } else {
-        /* If the port was tagged by the udev rules but isn't a primary or secondary,
-         * then ignore it to guard against race conditions if a device just happens
-         * to show up with more than two AT-capable ports.
-         */
-        ptype = MM_PORT_TYPE_IGNORED;
-    }
-
-    return mm_base_modem_grab_port (modem,
-                                    mm_port_probe_get_port_subsys (probe),
-                                    mm_port_probe_get_port_name (probe),
-                                    mm_port_probe_get_parent_path (probe),
-                                    ptype,
-                                    pflags,
-                                    error);
-}
-
 /*****************************************************************************/
 
 G_MODULE_EXPORT MMPlugin *
@@ -286,5 +238,4 @@ mm_plugin_longcheer_class_init (MMPluginLongcheerClass *klass)
     MMPluginClass *plugin_class = MM_PLUGIN_CLASS (klass);
 
     plugin_class->create_modem = create_modem;
-    plugin_class->grab_port = grab_port;
 }
