@@ -53,9 +53,7 @@ mm_common_sierra_grab_port (MMPlugin *self,
         pflags = MM_PORT_SERIAL_AT_FLAG_PRIMARY;
 
     return mm_base_modem_grab_port (modem,
-                                    mm_port_probe_get_port_subsys (probe),
-                                    mm_port_probe_get_port_name (probe),
-                                    mm_port_probe_get_parent_path (probe),
+                                    mm_port_probe_peek_port (probe),
                                     ptype,
                                     pflags,
                                     error);
@@ -79,23 +77,14 @@ mm_common_sierra_port_probe_list_is_icera (GList *probes)
 }
 
 typedef struct {
-    MMPortProbe *probe;
     MMPortSerialAt *port;
-    GCancellable *cancellable;
-    GSimpleAsyncResult *result;
     guint retries;
 } SierraCustomInitContext;
 
 static void
-sierra_custom_init_context_complete_and_free (SierraCustomInitContext *ctx)
+sierra_custom_init_context_free (SierraCustomInitContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-
-    if (ctx->cancellable)
-        g_object_unref (ctx->cancellable);
     g_object_unref (ctx->port);
-    g_object_unref (ctx->probe);
-    g_object_unref (ctx->result);
     g_slice_free (SierraCustomInitContext, ctx);
 }
 
@@ -104,31 +93,36 @@ mm_common_sierra_custom_init_finish (MMPortProbe *probe,
                                      GAsyncResult *result,
                                      GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error);
+    return g_task_propagate_boolean (G_TASK (result), error);
 }
 
-static void sierra_custom_init_step (SierraCustomInitContext *ctx);
+static void sierra_custom_init_step (GTask *task);
 
 static void
 gcap_ready (MMPortSerialAt *port,
             GAsyncResult *res,
-            SierraCustomInitContext *ctx)
+            GTask *task)
 {
+    MMPortProbe *probe;
+    SierraCustomInitContext *ctx;
     const gchar *response;
     GError *error = NULL;
 
+    probe = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     response = mm_port_serial_at_command_finish (port, res, &error);
     if (error) {
         /* If consumed all tries and the last error was a timeout, assume the
          * port is not AT */
         if (ctx->retries == 0 &&
             g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT)) {
-            mm_port_probe_set_result_at (ctx->probe, FALSE);
+            mm_port_probe_set_result_at (probe, FALSE);
         }
         /* If reported a hard parse error, this port is definitely not an AT
          * port, skip trying. */
         else if (g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_PARSE_FAILED)) {
-            mm_port_probe_set_result_at (ctx->probe, FALSE);
+            mm_port_probe_set_result_at (probe, FALSE);
             ctx->retries = 0;
         }
         /* Some Icera-based devices (eg, USB305) have an AT-style port that
@@ -136,16 +130,16 @@ gcap_ready (MMPortSerialAt *port,
          * the real AT ports do this too, so let a retry tag the port as
          * supported if it responds correctly later. */
         else if (g_error_matches (error, MM_MOBILE_EQUIPMENT_ERROR, MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN)) {
-            mm_port_probe_set_result_at (ctx->probe, FALSE);
+            mm_port_probe_set_result_at (probe, FALSE);
         }
 
         /* Just retry... */
-        sierra_custom_init_step (ctx);
+        sierra_custom_init_step (task);
         goto out;
     }
 
     /* A valid reply to ATI tells us this is an AT port already */
-    mm_port_probe_set_result_at (ctx->probe, TRUE);
+    mm_port_probe_set_result_at (probe, TRUE);
 
     /* Sierra APPx ports have limited AT command parsers that just reply with
      * "OK" to most commands.  These can sometimes be used for PPP while the
@@ -154,20 +148,20 @@ gcap_ready (MMPortSerialAt *port,
      * secondary APP ports.
      */
     if (strstr (response, "APP1")) {
-        g_object_set_data (G_OBJECT (ctx->probe), TAG_SIERRA_APP_PORT, GUINT_TO_POINTER (TRUE));
+        g_object_set_data (G_OBJECT (probe), TAG_SIERRA_APP_PORT, GUINT_TO_POINTER (TRUE));
 
         /* PPP-on-APP1-port whitelist */
         if (strstr (response, "C885") ||
             strstr (response, "USB 306") ||
             strstr (response, "MC8790"))
-            g_object_set_data (G_OBJECT (ctx->probe), TAG_SIERRA_APP1_PPP_OK, GUINT_TO_POINTER (TRUE));
+            g_object_set_data (G_OBJECT (probe), TAG_SIERRA_APP1_PPP_OK, GUINT_TO_POINTER (TRUE));
 
         /* For debugging: let users figure out if their device supports PPP
          * on the APP1 port or not.
          */
         if (getenv ("MM_SIERRA_APP1_PPP_OK")) {
             mm_dbg ("Sierra: APP1 PPP OK '%s'", response);
-            g_object_set_data (G_OBJECT (ctx->probe), TAG_SIERRA_APP1_PPP_OK, GUINT_TO_POINTER (TRUE));
+            g_object_set_data (G_OBJECT (probe), TAG_SIERRA_APP1_PPP_OK, GUINT_TO_POINTER (TRUE));
         }
     } else if (strstr (response, "APP2") ||
                strstr (response, "APP3") ||
@@ -175,11 +169,11 @@ gcap_ready (MMPortSerialAt *port,
         /* Additional APP ports don't support most AT commands, so they cannot
          * be used as the primary port.
          */
-        g_object_set_data (G_OBJECT (ctx->probe), TAG_SIERRA_APP_PORT, GUINT_TO_POINTER (TRUE));
+        g_object_set_data (G_OBJECT (probe), TAG_SIERRA_APP_PORT, GUINT_TO_POINTER (TRUE));
     }
 
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    sierra_custom_init_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 
 out:
     if (error)
@@ -187,22 +181,28 @@ out:
 }
 
 static void
-sierra_custom_init_step (SierraCustomInitContext *ctx)
+sierra_custom_init_step (GTask *task)
 {
+    SierraCustomInitContext *ctx;
+    GCancellable *cancellable;
+
+    ctx = g_task_get_task_data (task);
+    cancellable = g_task_get_cancellable (task);
+
     /* If cancelled, end */
-    if (g_cancellable_is_cancelled (ctx->cancellable)) {
+    if (g_cancellable_is_cancelled (cancellable)) {
         mm_dbg ("(Sierra) no need to keep on running custom init in '%s'",
                 mm_port_get_device (MM_PORT (ctx->port)));
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        sierra_custom_init_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
     if (ctx->retries == 0) {
         mm_dbg ("(Sierra) Couldn't get port type hints from '%s'",
                 mm_port_get_device (MM_PORT (ctx->port)));
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        sierra_custom_init_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -213,9 +213,9 @@ sierra_custom_init_step (SierraCustomInitContext *ctx)
         3,
         FALSE, /* raw */
         FALSE, /* allow_cached */
-        ctx->cancellable,
+        cancellable,
         (GAsyncReadyCallback)gcap_ready,
-        ctx);
+        task);
 }
 
 void
@@ -226,18 +226,17 @@ mm_common_sierra_custom_init (MMPortProbe *probe,
                               gpointer user_data)
 {
     SierraCustomInitContext *ctx;
+    GTask *task;
 
     ctx = g_slice_new (SierraCustomInitContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (probe),
-                                             callback,
-                                             user_data,
-                                             mm_common_sierra_custom_init);
-    ctx->probe = g_object_ref (probe);
     ctx->port = g_object_ref (port);
-    ctx->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
     ctx->retries = 3;
 
-    sierra_custom_init_step (ctx);
+    task = g_task_new (probe, cancellable, callback, user_data);
+    g_task_set_check_cancellable (task, FALSE);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)sierra_custom_init_context_free);
+
+    sierra_custom_init_step (task);
 }
 
 /*****************************************************************************/
@@ -248,22 +247,21 @@ mm_common_sierra_modem_power_up_finish (MMIfaceModem *self,
                                         GAsyncResult *res,
                                         GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static gboolean
-sierra_power_up_wait_cb (GSimpleAsyncResult *result)
+sierra_power_up_wait_cb (GTask *task)
 {
-    g_simple_async_result_set_op_res_gboolean (result, TRUE);
-    g_simple_async_result_complete (result);
-    g_object_unref (result);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
     return G_SOURCE_REMOVE;
 }
 
 static void
 cfun_enable_ready (MMBaseModem *self,
                    GAsyncResult *res,
-                   GSimpleAsyncResult *simple)
+                   GTask *task)
 {
     GError *error = NULL;
     guint i;
@@ -271,9 +269,8 @@ cfun_enable_ready (MMBaseModem *self,
     gboolean is_new_sierra = FALSE;
 
     if (!mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error)) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -292,24 +289,23 @@ cfun_enable_ready (MMBaseModem *self,
         }
     }
 
-    /* The modem object will be valid in the callback as 'result' keeps a
+    /* The modem object will be valid in the callback as 'task' keeps a
      * reference to it. */
-    g_timeout_add_seconds (is_new_sierra ? 5 : 10, (GSourceFunc)sierra_power_up_wait_cb, simple);
+    g_timeout_add_seconds (is_new_sierra ? 5 : 10, (GSourceFunc)sierra_power_up_wait_cb, task);
 }
 
 static void
 pcstate_enable_ready (MMBaseModem *self,
                       GAsyncResult *res,
-                      GSimpleAsyncResult *simple)
+                      GTask *task)
 {
     /* Ignore errors for now; we're not sure if all Sierra CDMA devices support
      * at!pcstate.
      */
     mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, NULL);
 
-    g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 void
@@ -317,12 +313,9 @@ mm_common_sierra_modem_power_up (MMIfaceModem *self,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        mm_common_sierra_modem_power_up);
+    task = g_task_new (self, NULL, callback, user_data);
 
     /* For CDMA modems, run !pcstate */
     if (mm_iface_modem_is_cdma_only (self)) {
@@ -331,7 +324,7 @@ mm_common_sierra_modem_power_up (MMIfaceModem *self,
                                   5,
                                   FALSE,
                                   (GAsyncReadyCallback)pcstate_enable_ready,
-                                  result);
+                                  task);
         return;
     }
 
@@ -346,7 +339,7 @@ mm_common_sierra_modem_power_up (MMIfaceModem *self,
                               10,
                               FALSE,
                               (GAsyncReadyCallback)cfun_enable_ready,
-                              result);
+                              task);
 }
 
 /*****************************************************************************/
@@ -357,33 +350,38 @@ mm_common_sierra_load_power_state_finish (MMIfaceModem *self,
                                           GAsyncResult *res,
                                           GError **error)
 {
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return MM_MODEM_POWER_STATE_UNKNOWN;
+    GError *inner_error = NULL;
+    gssize value;
 
-    return (MMModemPowerState)GPOINTER_TO_UINT (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    value = g_task_propagate_int (G_TASK (res), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return MM_MODEM_POWER_STATE_UNKNOWN;
+    }
+    return (MMModemPowerState)value;
 }
 
 static void
 parent_load_power_state_ready (MMIfaceModem *self,
                                GAsyncResult *res,
-                               GSimpleAsyncResult *simple)
+                               GTask *task)
 {
     GError *error = NULL;
     MMModemPowerState state;
 
     state = iface_modem_parent->load_power_state_finish (self, res, &error);
     if (error)
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gpointer (simple, GUINT_TO_POINTER (state), NULL);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_int (task, state);
+
+    g_object_unref (task);
 }
 
 static void
 pcstate_query_ready (MMBaseModem *self,
                      GAsyncResult *res,
-                     GSimpleAsyncResult *simple)
+                     GTask *task)
 {
     const gchar *result;
     guint state;
@@ -391,40 +389,38 @@ pcstate_query_ready (MMBaseModem *self,
 
     result = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
     if (!result) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Parse power state reply */
     result = mm_strip_tag (result, "!PCSTATE:");
     if (!mm_get_uint_from_str (result, &state)) {
-        g_simple_async_result_set_error (simple,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Failed to parse !PCSTATE response '%s'",
-                                         result);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Failed to parse !PCSTATE response '%s'",
+                                 result);
     } else {
         switch (state) {
         case 0:
-            g_simple_async_result_set_op_res_gpointer (simple, GUINT_TO_POINTER (MM_MODEM_POWER_STATE_LOW), NULL);
+            g_task_return_int (task, MM_MODEM_POWER_STATE_LOW);
             break;
         case 1:
-            g_simple_async_result_set_op_res_gpointer (simple, GUINT_TO_POINTER (MM_MODEM_POWER_STATE_ON), NULL);
+            g_task_return_int (task, MM_MODEM_POWER_STATE_ON);
             break;
         default:
-            g_simple_async_result_set_error (simple,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_FAILED,
-                                             "Unhandled power state: '%u'",
-                                             state);
+            g_task_return_new_error (task,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Unhandled power state: '%u'",
+                                     state);
             break;
         }
     }
 
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 void
@@ -432,12 +428,9 @@ mm_common_sierra_load_power_state (MMIfaceModem *self,
                                    GAsyncReadyCallback callback,
                                    gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        mm_common_sierra_load_power_state);
+    task = g_task_new (self, NULL, callback, user_data);
 
     /* Check power state with AT!PCSTATE? */
     if (mm_iface_modem_is_cdma_only (self)) {
@@ -446,14 +439,14 @@ mm_common_sierra_load_power_state (MMIfaceModem *self,
                                   3,
                                   FALSE,
                                   (GAsyncReadyCallback)pcstate_query_ready,
-                                  result);
+                                  task);
         return;
     }
 
     /* Otherwise run parent's */
     iface_modem_parent->load_power_state (self,
                                           (GAsyncReadyCallback)parent_load_power_state_ready,
-                                          result);
+                                          task);
 }
 
 /*****************************************************************************/
@@ -494,7 +487,7 @@ mm_common_sierra_setup_ports (MMBroadbandModem *self)
     ports[0] = mm_base_modem_peek_port_primary (MM_BASE_MODEM (self));
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
