@@ -32,17 +32,41 @@
 #include "mm-base-modem-at.h"
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-3gpp.h"
+#include "mm-iface-modem-location.h"
+#include "mm-iface-modem-voice.h"
+#include "mm-shared-simtech.h"
 #include "mm-broadband-modem-simtech.h"
 
-static void iface_modem_init (MMIfaceModem *iface);
-static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
+static void iface_modem_init          (MMIfaceModem         *iface);
+static void iface_modem_3gpp_init     (MMIfaceModem3gpp     *iface);
+static void iface_modem_location_init (MMIfaceModemLocation *iface);
+static void iface_modem_voice_init    (MMIfaceModemVoice    *iface);
+static void shared_simtech_init       (MMSharedSimtech      *iface);
 
-static MMIfaceModem *iface_modem_parent;
-static MMIfaceModem3gpp *iface_modem_3gpp_parent;
+static MMIfaceModem         *iface_modem_parent;
+static MMIfaceModem3gpp     *iface_modem_3gpp_parent;
+static MMIfaceModemLocation *iface_modem_location_parent;
+static MMIfaceModemVoice    *iface_modem_voice_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemSimtech, mm_broadband_modem_simtech, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init))
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_LOCATION, iface_modem_location_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_VOICE, iface_modem_voice_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_SHARED_SIMTECH, shared_simtech_init))
+
+typedef enum {
+    FEATURE_SUPPORT_UNKNOWN,
+    FEATURE_NOT_SUPPORTED,
+    FEATURE_SUPPORTED
+} FeatureSupport;
+
+struct _MMBroadbandModemSimtechPrivate {
+    FeatureSupport  cnsmod_support;
+    FeatureSupport  autocsq_support;
+    GRegex         *cnsmod_regex;
+    GRegex         *csq_regex;
+};
 
 /*****************************************************************************/
 /* Setup/Cleanup unsolicited events (3GPP interface) */
@@ -50,22 +74,19 @@ G_DEFINE_TYPE_EXTENDED (MMBroadbandModemSimtech, mm_broadband_modem_simtech, MM_
 static MMModemAccessTechnology
 simtech_act_to_mm_act (int nsmod)
 {
-    if (nsmod == 1)
-        return MM_MODEM_ACCESS_TECHNOLOGY_GSM;
-    else if (nsmod == 2)
-        return MM_MODEM_ACCESS_TECHNOLOGY_GPRS;
-    else if (nsmod == 3)
-        return MM_MODEM_ACCESS_TECHNOLOGY_EDGE;
-    else if (nsmod == 4)
-        return MM_MODEM_ACCESS_TECHNOLOGY_UMTS;
-    else if (nsmod == 5)
-        return MM_MODEM_ACCESS_TECHNOLOGY_HSDPA;
-    else if (nsmod == 6)
-        return MM_MODEM_ACCESS_TECHNOLOGY_HSUPA;
-    else if (nsmod == 7)
-        return MM_MODEM_ACCESS_TECHNOLOGY_HSPA;
+    static const MMModemAccessTechnology simtech_act_to_mm_act_map[] = {
+        [0] = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN,
+        [1] = MM_MODEM_ACCESS_TECHNOLOGY_GSM,
+        [2] = MM_MODEM_ACCESS_TECHNOLOGY_GPRS,
+        [3] = MM_MODEM_ACCESS_TECHNOLOGY_EDGE,
+        [4] = MM_MODEM_ACCESS_TECHNOLOGY_UMTS,
+        [5] = MM_MODEM_ACCESS_TECHNOLOGY_HSDPA,
+        [6] = MM_MODEM_ACCESS_TECHNOLOGY_HSUPA,
+        [7] = MM_MODEM_ACCESS_TECHNOLOGY_HSPA,
+        [8] = MM_MODEM_ACCESS_TECHNOLOGY_LTE,
+    };
 
-    return MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
+    return (nsmod < G_N_ELEMENTS (simtech_act_to_mm_act_map) ? simtech_act_to_mm_act_map[nsmod] : MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
 }
 
 static void
@@ -84,20 +105,34 @@ simtech_tech_changed (MMPortSerialAt *port,
     g_free (str);
 }
 
+static void
+simtech_signal_changed (MMPortSerialAt *port,
+                        GMatchInfo *match_info,
+                        MMBroadbandModemSimtech *self)
+{
+    guint quality = 0;
+
+    if (!mm_get_uint_from_match_info (match_info, 1, &quality))
+        return;
+
+    if (quality != 99)
+        quality = CLAMP (quality, 0, 31) * 100 / 31;
+    else
+        quality = 0;
+
+    mm_iface_modem_update_signal_quality (MM_IFACE_MODEM (self), quality);
+}
+
 static void
 set_unsolicited_events_handlers (MMBroadbandModemSimtech *self,
                                  gboolean enable)
 {
     MMPortSerialAt *ports[2];
     guint i;
-    GRegex *regex;
 
     ports[0] = mm_base_modem_peek_port_primary (MM_BASE_MODEM (self));
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
-    regex = g_regex_new ("\\r\\n\\+CNSMOD:\\s*(\\d)\\r\\n",
-                         G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
-
     /* Enable unsolicited events in given port */
     for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
@@ -106,13 +141,19 @@ set_unsolicited_events_handlers (MMBroadbandModemSimtech *self,
         /* Access technology related */
         mm_port_serial_at_add_unsolicited_msg_handler (
             ports[i],
-            regex,
+            self->priv->cnsmod_regex,
             enable ? (MMPortSerialAtUnsolicitedMsgFn)simtech_tech_changed : NULL,
             enable ? self : NULL,
             NULL);
-    }
 
-    g_regex_unref (regex);
+        /* Signal quality related */
+        mm_port_serial_at_add_unsolicited_msg_handler (
+            ports[i],
+            self->priv->csq_regex,
+            enable ? (MMPortSerialAtUnsolicitedMsgFn)simtech_signal_changed : NULL,
+            enable ? self : NULL,
+            NULL);
+    }
 }
 
 static gboolean
@@ -182,7 +223,21 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp    *self,
 }
 
 /*****************************************************************************/
-/* Enabling unsolicited events (3GPP interface) */
+/* Enable unsolicited events (3GPP interface) */
+
+typedef enum {
+    ENABLE_UNSOLICITED_EVENTS_STEP_FIRST,
+    ENABLE_UNSOLICITED_EVENTS_STEP_PARENT,
+    ENABLE_UNSOLICITED_EVENTS_STEP_CHECK_SUPPORT_CNSMOD,
+    ENABLE_UNSOLICITED_EVENTS_STEP_ENABLE_CNSMOD,
+    ENABLE_UNSOLICITED_EVENTS_STEP_CHECK_SUPPORT_AUTOCSQ,
+    ENABLE_UNSOLICITED_EVENTS_STEP_ENABLE_AUTOCSQ,
+    ENABLE_UNSOLICITED_EVENTS_STEP_LAST,
+} EnableUnsolicitedEventsStep;
+
+typedef struct {
+    EnableUnsolicitedEventsStep step;
+} EnableUnsolicitedEventsContext;
 
 static gboolean
 modem_3gpp_enable_unsolicited_events_finish (MMIfaceModem3gpp  *self,
@@ -192,35 +247,113 @@ modem_3gpp_enable_unsolicited_events_finish (MMIfaceModem3gpp  *self,
     return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static void
-own_enable_unsolicited_events_ready (MMBaseModem  *self,
-                                     GAsyncResult *res,
-                                     GTask        *task)
-{
-    GError *error = NULL;
+static void enable_unsolicited_events_context_step (GTask *task);
 
-    mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
-    if (error)
-        g_task_return_error (task, error);
-    else
-        g_task_return_boolean (task, TRUE);
-    g_object_unref (task);
+static void
+autocsq_set_enabled_ready (MMBaseModem  *self,
+                           GAsyncResult *res,
+                           GTask        *task)
+{
+    EnableUnsolicitedEventsContext *ctx;
+    GError                         *error = NULL;
+    gboolean                        csq_urcs_enabled = FALSE;
+
+    ctx = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (self, res, &error)) {
+        mm_dbg ("Couldn't enable automatic signal quality reporting: %s", error->message);
+        g_error_free (error);
+    } else
+        csq_urcs_enabled = TRUE;
+
+    /* Disable access technology polling if we can use the +CSQ URCs */
+    g_object_set (self,
+                  MM_IFACE_MODEM_PERIODIC_SIGNAL_CHECK_DISABLED, csq_urcs_enabled,
+                  NULL);
+
+    /* go to next step */
+    ctx->step++;
+    enable_unsolicited_events_context_step (task);
 }
 
-static const MMBaseModemAtCommand unsolicited_enable_sequence[] = {
-    /* Autoreport access technology changes */
-    { "+CNSMOD=1",    5, FALSE, NULL },
-    /* Autoreport CSQ (first arg), and only report when it changes (second arg) */
-    { "+AUTOCSQ=1,1", 5, FALSE, NULL },
-    { NULL }
-};
+static void
+autocsq_test_ready (MMBaseModem  *_self,
+                    GAsyncResult *res,
+                    GTask        *task)
+{
+    MMBroadbandModemSimtech        *self;
+    EnableUnsolicitedEventsContext *ctx;
+
+    self = MM_BROADBAND_MODEM_SIMTECH (_self);
+    ctx  = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (_self, res, NULL))
+        self->priv->autocsq_support = FEATURE_NOT_SUPPORTED;
+    else
+        self->priv->autocsq_support = FEATURE_SUPPORTED;
+
+    /* go to next step */
+    ctx->step++;
+    enable_unsolicited_events_context_step (task);
+}
+
+static void
+cnsmod_set_enabled_ready (MMBaseModem  *self,
+                          GAsyncResult *res,
+                          GTask        *task)
+{
+    EnableUnsolicitedEventsContext *ctx;
+    GError                         *error = NULL;
+    gboolean                        cnsmod_urcs_enabled = FALSE;
+
+    ctx = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (self, res, &error)) {
+        mm_dbg ("Couldn't enable automatic access technology reporting: %s", error->message);
+        g_error_free (error);
+    } else
+        cnsmod_urcs_enabled = TRUE;
+
+    /* Disable access technology polling if we can use the +CNSMOD URCs */
+    g_object_set (self,
+                  MM_IFACE_MODEM_PERIODIC_ACCESS_TECH_CHECK_DISABLED, cnsmod_urcs_enabled,
+                  NULL);
+
+    /* go to next step */
+    ctx->step++;
+    enable_unsolicited_events_context_step (task);
+}
+
+static void
+cnsmod_test_ready (MMBaseModem  *_self,
+                   GAsyncResult *res,
+                   GTask        *task)
+{
+    MMBroadbandModemSimtech        *self;
+    EnableUnsolicitedEventsContext *ctx;
+
+    self = MM_BROADBAND_MODEM_SIMTECH (_self);
+    ctx  = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (_self, res, NULL))
+        self->priv->cnsmod_support = FEATURE_NOT_SUPPORTED;
+    else
+        self->priv->cnsmod_support = FEATURE_SUPPORTED;
+
+    /* go to next step */
+    ctx->step++;
+    enable_unsolicited_events_context_step (task);
+}
 
 static void
 parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                         GAsyncResult     *res,
                                         GTask            *task)
 {
-    GError *error = NULL;
+    EnableUnsolicitedEventsContext *ctx;
+    GError                         *error = NULL;
+
+    ctx = g_task_get_task_data (task);
 
     if (!iface_modem_3gpp_parent->enable_unsolicited_events_finish (self, res, &error)) {
         g_task_return_error (task, error);
@@ -228,16 +361,91 @@ parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
         return;
     }
 
-    /* Our own enable now */
-    mm_base_modem_at_sequence_full (
-        MM_BASE_MODEM (self),
-        mm_base_modem_peek_port_primary (MM_BASE_MODEM (self)),
-        unsolicited_enable_sequence,
-        NULL, /* response_processor_context */
-        NULL, /* response_processor_context_free */
-        NULL, /* cancellable */
-        (GAsyncReadyCallback)own_enable_unsolicited_events_ready,
-        task);
+    /* go to next step */
+    ctx->step++;
+    enable_unsolicited_events_context_step (task);
+}
+
+static void
+enable_unsolicited_events_context_step (GTask *task)
+{
+    MMBroadbandModemSimtech        *self;
+    EnableUnsolicitedEventsContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx  = g_task_get_task_data (task);
+
+    switch (ctx->step) {
+    case ENABLE_UNSOLICITED_EVENTS_STEP_FIRST:
+        /* fall down to next step */
+        ctx->step++;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_PARENT:
+        iface_modem_3gpp_parent->enable_unsolicited_events (
+            MM_IFACE_MODEM_3GPP (self),
+            (GAsyncReadyCallback)parent_enable_unsolicited_events_ready,
+            task);
+        return;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_CHECK_SUPPORT_CNSMOD:
+        if (self->priv->cnsmod_support == FEATURE_SUPPORT_UNKNOWN) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      "+CNSMOD=?",
+                                      3,
+                                      TRUE,
+                                      (GAsyncReadyCallback)cnsmod_test_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_ENABLE_CNSMOD:
+        if (self->priv->cnsmod_support == FEATURE_SUPPORTED) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      /* Autoreport +CNSMOD when it changes */
+                                      "+CNSMOD=1",
+                                      20,
+                                      FALSE,
+                                      (GAsyncReadyCallback)cnsmod_set_enabled_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_CHECK_SUPPORT_AUTOCSQ:
+        if (self->priv->autocsq_support == FEATURE_SUPPORT_UNKNOWN) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      "+AUTOCSQ=?",
+                                      3,
+                                      TRUE,
+                                      (GAsyncReadyCallback)autocsq_test_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_ENABLE_AUTOCSQ:
+        if (self->priv->autocsq_support == FEATURE_SUPPORTED) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      /* Autoreport+ CSQ (first arg), and only report when it changes (second arg) */
+                                      "+AUTOCSQ=1,1",
+                                      20,
+                                      FALSE,
+                                      (GAsyncReadyCallback)autocsq_set_enabled_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case ENABLE_UNSOLICITED_EVENTS_STEP_LAST:
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
+        return;
+    }
 }
 
 static void
@@ -245,80 +453,173 @@ modem_3gpp_enable_unsolicited_events (MMIfaceModem3gpp    *self,
                                       GAsyncReadyCallback  callback,
                                       gpointer             user_data)
 {
-    /* Chain up parent's enable */
-    iface_modem_3gpp_parent->enable_unsolicited_events (
-        self,
-        (GAsyncReadyCallback)parent_enable_unsolicited_events_ready,
-        g_task_new (self, NULL, callback, user_data));
+    EnableUnsolicitedEventsContext *ctx;
+    GTask                          *task;
+
+    task = g_task_new (self, NULL, callback, user_data);
+
+    ctx = g_new (EnableUnsolicitedEventsContext, 1);
+    ctx->step = ENABLE_UNSOLICITED_EVENTS_STEP_FIRST;
+    g_task_set_task_data (task, ctx, g_free);
+
+    enable_unsolicited_events_context_step (task);
 }
 
 /*****************************************************************************/
-/* Disabling unsolicited events (3GPP interface) */
+/* Disable unsolicited events (3GPP interface) */
+
+typedef enum {
+    DISABLE_UNSOLICITED_EVENTS_STEP_FIRST,
+    DISABLE_UNSOLICITED_EVENTS_STEP_DISABLE_AUTOCSQ,
+    DISABLE_UNSOLICITED_EVENTS_STEP_DISABLE_CNSMOD,
+    DISABLE_UNSOLICITED_EVENTS_STEP_PARENT,
+    DISABLE_UNSOLICITED_EVENTS_STEP_LAST,
+} DisableUnsolicitedEventsStep;
+
+typedef struct {
+    DisableUnsolicitedEventsStep step;
+} DisableUnsolicitedEventsContext;
 
 static gboolean
-modem_3gpp_disable_unsolicited_events_finish (MMIfaceModem3gpp *self,
-                                              GAsyncResult     *res,
-                                              GError          **error)
+modem_3gpp_disable_unsolicited_events_finish (MMIfaceModem3gpp  *self,
+                                              GAsyncResult      *res,
+                                              GError           **error)
 {
     return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static const MMBaseModemAtCommand unsolicited_disable_sequence[] = {
-    { "+CNSMOD=0",  3, FALSE, NULL },
-    { "+AUTOCSQ=0", 3, FALSE, NULL },
-    { NULL }
-};
+static void disable_unsolicited_events_context_step (GTask *task);
 
 static void
 parent_disable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                          GAsyncResult     *res,
                                          GTask            *task)
 {
-    GError *error = NULL;
+    DisableUnsolicitedEventsContext *ctx;
+    GError                         *error = NULL;
 
-    if (!iface_modem_3gpp_parent->disable_unsolicited_events_finish (self, res, &error))
-        g_task_return_error (task, error);
-    else
-        g_task_return_boolean (task, TRUE);
-    g_object_unref (task);
-}
+    ctx = g_task_get_task_data (task);
 
-static void
-own_disable_unsolicited_events_ready (MMBaseModem  *self,
-                                      GAsyncResult *res,
-                                      GTask        *task)
-{
-    GError *error = NULL;
-
-    mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
-    if (error) {
+    if (!iface_modem_3gpp_parent->disable_unsolicited_events_finish (self, res, &error)) {
         g_task_return_error (task, error);
         g_object_unref (task);
         return;
     }
 
-    /* Next, chain up parent's disable */
-    iface_modem_3gpp_parent->disable_unsolicited_events (
-        MM_IFACE_MODEM_3GPP (self),
-        (GAsyncReadyCallback)parent_disable_unsolicited_events_ready,
-        task);
+    /* go to next step */
+    ctx->step++;
+    disable_unsolicited_events_context_step (task);
 }
 
 static void
-modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp *self,
-                                       GAsyncReadyCallback callback,
-                                       gpointer user_data)
+cnsmod_set_disabled_ready (MMBaseModem  *self,
+                           GAsyncResult *res,
+                           GTask        *task)
 {
-    /* Our own disable first */
-    mm_base_modem_at_sequence_full (
-        MM_BASE_MODEM (self),
-        mm_base_modem_peek_port_primary (MM_BASE_MODEM (self)),
-        unsolicited_disable_sequence,
-        NULL, /* response_processor_context */
-        NULL, /* response_processor_context_free */
-        NULL, /* cancellable */
-        (GAsyncReadyCallback)own_disable_unsolicited_events_ready,
-        g_task_new (self, NULL, callback, user_data));
+    DisableUnsolicitedEventsContext *ctx;
+    GError                          *error = NULL;
+
+    ctx = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (self, res, &error)) {
+        mm_dbg ("Couldn't disable automatic access technology reporting: %s", error->message);
+        g_error_free (error);
+    }
+
+    /* go to next step */
+    ctx->step++;
+    disable_unsolicited_events_context_step (task);
+}
+
+static void
+autocsq_set_disabled_ready (MMBaseModem  *self,
+                            GAsyncResult *res,
+                            GTask        *task)
+{
+    DisableUnsolicitedEventsContext *ctx;
+    GError                          *error = NULL;
+
+    ctx = g_task_get_task_data (task);
+
+    if (!mm_base_modem_at_command_finish (self, res, &error)) {
+        mm_dbg ("Couldn't disable automatic signal quality reporting: %s", error->message);
+        g_error_free (error);
+    }
+
+    /* go to next step */
+    ctx->step++;
+    disable_unsolicited_events_context_step (task);
+}
+
+static void
+disable_unsolicited_events_context_step (GTask *task)
+{
+    MMBroadbandModemSimtech         *self;
+    DisableUnsolicitedEventsContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx  = g_task_get_task_data (task);
+
+    switch (ctx->step) {
+    case DISABLE_UNSOLICITED_EVENTS_STEP_FIRST:
+        /* fall down to next step */
+        ctx->step++;
+
+    case DISABLE_UNSOLICITED_EVENTS_STEP_DISABLE_AUTOCSQ:
+        if (self->priv->autocsq_support == FEATURE_SUPPORTED) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      "+AUTOCSQ=0",
+                                      20,
+                                      FALSE,
+                                      (GAsyncReadyCallback)autocsq_set_disabled_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case DISABLE_UNSOLICITED_EVENTS_STEP_DISABLE_CNSMOD:
+        if (self->priv->cnsmod_support == FEATURE_SUPPORTED) {
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
+                                      "+CNSMOD=0",
+                                      20,
+                                      FALSE,
+                                      (GAsyncReadyCallback)cnsmod_set_disabled_ready,
+                                      task);
+            return;
+        }
+        /* fall down to next step */
+        ctx->step++;
+
+    case DISABLE_UNSOLICITED_EVENTS_STEP_PARENT:
+        iface_modem_3gpp_parent->disable_unsolicited_events (
+            MM_IFACE_MODEM_3GPP (self),
+            (GAsyncReadyCallback)parent_disable_unsolicited_events_ready,
+            task);
+        return;
+
+    case DISABLE_UNSOLICITED_EVENTS_STEP_LAST:
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
+        return;
+    }
+}
+
+static void
+modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp    *self,
+                                      GAsyncReadyCallback  callback,
+                                      gpointer             user_data)
+{
+    DisableUnsolicitedEventsContext *ctx;
+    GTask                          *task;
+
+    task = g_task_new (self, NULL, callback, user_data);
+
+    ctx = g_new (DisableUnsolicitedEventsContext, 1);
+    ctx->step = DISABLE_UNSOLICITED_EVENTS_STEP_FIRST;
+    g_task_set_task_data (task, ctx, g_free);
+
+    disable_unsolicited_events_context_step (task);
 }
 
 /*****************************************************************************/
@@ -377,21 +678,31 @@ cnsmod_query_ready (MMBaseModem  *self,
 }
 
 static void
-load_access_technologies (MMIfaceModem        *self,
+load_access_technologies (MMIfaceModem        *_self,
                           GAsyncReadyCallback  callback,
                           gpointer             user_data)
 {
-    GTask *task;
+    MMBroadbandModemSimtech *self;
+    GTask                   *task;
 
+    self = MM_BROADBAND_MODEM_SIMTECH (_self);
     task = g_task_new (self, NULL, callback, user_data);
 
     /* Launch query only for 3GPP modems */
-    if (!mm_iface_modem_is_3gpp (self)) {
+    if (!mm_iface_modem_is_3gpp (_self)) {
         g_task_return_int (task, MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
         g_object_unref (task);
         return;
     }
 
+    g_assert (self->priv->cnsmod_support != FEATURE_SUPPORT_UNKNOWN);
+    if (self->priv->cnsmod_support == FEATURE_NOT_SUPPORTED) {
+        g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_UNSUPPORTED,
+                                 "Loading access technologies with +CNSMOD is not supported");
+        g_object_unref (task);
+        return;
+    }
+
     mm_base_modem_at_command (
         MM_BASE_MODEM (self),
         "AT+CNSMOD?",
@@ -401,6 +712,85 @@ load_access_technologies (MMIfaceModem        *self,
         task);
 }
 
+/*****************************************************************************/
+/* Load signal quality (Modem interface) */
+
+static guint
+load_signal_quality_finish (MMIfaceModem  *self,
+                            GAsyncResult  *res,
+                            GError       **error)
+{
+    gssize value;
+
+    value = g_task_propagate_int (G_TASK (res), error);
+    return value < 0 ? 0 : value;
+}
+
+static void
+csq_query_ready (MMBaseModem  *self,
+                 GAsyncResult *res,
+                 GTask        *task)
+{
+    const gchar *response, *p;
+    GError      *error = NULL;
+    gint         quality;
+    gint         ber;
+
+    response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
+    if (!response) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+
+    /* Given that we may have enabled AUTOCSQ support, it is totally possible
+     * to get an empty string at this point, because the +CSQ reply may have
+     * been processed as an URC already. If we ever see this, we should not return
+     * an error, because that would reset the reported signal quality to 0 :/
+     * So, in this case, return the last cached signal quality value. */
+    if (!response[0]) {
+        g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_IN_PROGRESS,
+                                 "already refreshed via URCs");
+        g_object_unref (task);
+        return;
+    }
+
+    p = mm_strip_tag (response, "+CSQ:");
+    if (sscanf (p, "%d, %d", &quality, &ber)) {
+        if (quality != 99)
+            quality = CLAMP (quality, 0, 31) * 100 / 31;
+        else
+            quality = 0;
+        g_task_return_int (task, quality);
+        g_object_unref (task);
+        return;
+    }
+
+    g_task_return_new_error (task,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_FAILED,
+                             "Could not parse signal quality results");
+    g_object_unref (task);
+}
+
+static void
+load_signal_quality (MMIfaceModem        *self,
+                     GAsyncReadyCallback  callback,
+                     gpointer             user_data)
+{
+    GTask *task;
+
+    task = g_task_new (self, NULL, callback, user_data);
+
+    mm_base_modem_at_command (
+        MM_BASE_MODEM (self),
+        "+CSQ",
+        3,
+        FALSE,
+        (GAsyncReadyCallback)csq_query_ready,
+        task);
+}
+
 /*****************************************************************************/
 /* Load supported modes (Modem interface) */
 
@@ -819,12 +1209,53 @@ mm_broadband_modem_simtech_new (const gchar *device,
                          MM_BASE_MODEM_PLUGIN, plugin,
                          MM_BASE_MODEM_VENDOR_ID, vendor_id,
                          MM_BASE_MODEM_PRODUCT_ID, product_id,
+                         MM_BROADBAND_MODEM_INDICATORS_DISABLED, TRUE,
                          NULL);
 }
 
 static void
 mm_broadband_modem_simtech_init (MMBroadbandModemSimtech *self)
 {
+    /* Initialize private data */
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+                                              MM_TYPE_BROADBAND_MODEM_SIMTECH,
+                                              MMBroadbandModemSimtechPrivate);
+
+    self->priv->cnsmod_support = FEATURE_SUPPORT_UNKNOWN;
+    self->priv->autocsq_support = FEATURE_SUPPORT_UNKNOWN;
+
+    self->priv->cnsmod_regex = g_regex_new ("\\r\\n\\+CNSMOD:\\s*(\\d+)\\r\\n",
+                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+    self->priv->csq_regex    = g_regex_new ("\\r\\n\\+CSQ:\\s*(\\d+),(\\d+)\\r\\n",
+                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+}
+
+static void
+finalize (GObject *object)
+{
+    MMBroadbandModemSimtech *self = MM_BROADBAND_MODEM_SIMTECH (object);
+
+    g_regex_unref (self->priv->cnsmod_regex);
+    g_regex_unref (self->priv->csq_regex);
+
+    G_OBJECT_CLASS (mm_broadband_modem_simtech_parent_class)->finalize (object);
+}
+
+static void
+iface_modem_init (MMIfaceModem *iface)
+{
+    iface_modem_parent = g_type_interface_peek_parent (iface);
+
+    iface->load_signal_quality = load_signal_quality;
+    iface->load_signal_quality_finish = load_signal_quality_finish;
+    iface->load_access_technologies = load_access_technologies;
+    iface->load_access_technologies_finish = load_access_technologies_finish;
+    iface->load_supported_modes = load_supported_modes;
+    iface->load_supported_modes_finish = load_supported_modes_finish;
+    iface->load_current_modes = load_current_modes;
+    iface->load_current_modes_finish = load_current_modes_finish;
+    iface->set_current_modes = set_current_modes;
+    iface->set_current_modes_finish = set_current_modes_finish;
 }
 
 static void
@@ -844,24 +1275,68 @@ iface_modem_3gpp_init (MMIfaceModem3gpp *iface)
 }
 
 static void
-iface_modem_init (MMIfaceModem *iface)
+iface_modem_location_init (MMIfaceModemLocation *iface)
 {
-    iface_modem_parent = g_type_interface_peek_parent (iface);
+    iface_modem_location_parent = g_type_interface_peek_parent (iface);
 
-    iface->load_access_technologies = load_access_technologies;
-    iface->load_access_technologies_finish = load_access_technologies_finish;
-    iface->load_supported_modes = load_supported_modes;
-    iface->load_supported_modes_finish = load_supported_modes_finish;
-    iface->load_current_modes = load_current_modes;
-    iface->load_current_modes_finish = load_current_modes_finish;
-    iface->set_current_modes = set_current_modes;
-    iface->set_current_modes_finish = set_current_modes_finish;
+    iface->load_capabilities                 = mm_shared_simtech_location_load_capabilities;
+    iface->load_capabilities_finish          = mm_shared_simtech_location_load_capabilities_finish;
+    iface->enable_location_gathering         = mm_shared_simtech_enable_location_gathering;
+    iface->enable_location_gathering_finish  = mm_shared_simtech_enable_location_gathering_finish;
+    iface->disable_location_gathering        = mm_shared_simtech_disable_location_gathering;
+    iface->disable_location_gathering_finish = mm_shared_simtech_disable_location_gathering_finish;
+}
+
+static MMIfaceModemLocation *
+peek_parent_location_interface (MMSharedSimtech *self)
+{
+    return iface_modem_location_parent;
+}
+
+static void
+iface_modem_voice_init (MMIfaceModemVoice *iface)
+{
+    iface_modem_voice_parent = g_type_interface_peek_parent (iface);
+
+    iface->check_support                     = mm_shared_simtech_voice_check_support;
+    iface->check_support_finish              = mm_shared_simtech_voice_check_support_finish;
+    iface->enable_unsolicited_events         = mm_shared_simtech_voice_enable_unsolicited_events;
+    iface->enable_unsolicited_events_finish  = mm_shared_simtech_voice_enable_unsolicited_events_finish;
+    iface->disable_unsolicited_events        = mm_shared_simtech_voice_disable_unsolicited_events;
+    iface->disable_unsolicited_events_finish = mm_shared_simtech_voice_disable_unsolicited_events_finish;
+    iface->setup_unsolicited_events          = mm_shared_simtech_voice_setup_unsolicited_events;
+    iface->setup_unsolicited_events_finish   = mm_shared_simtech_voice_setup_unsolicited_events_finish;
+    iface->cleanup_unsolicited_events        = mm_shared_simtech_voice_cleanup_unsolicited_events;
+    iface->cleanup_unsolicited_events_finish = mm_shared_simtech_voice_cleanup_unsolicited_events_finish;
+    iface->setup_in_call_audio_channel          = mm_shared_simtech_voice_setup_in_call_audio_channel;
+    iface->setup_in_call_audio_channel_finish   = mm_shared_simtech_voice_setup_in_call_audio_channel_finish;
+    iface->cleanup_in_call_audio_channel        = mm_shared_simtech_voice_cleanup_in_call_audio_channel;
+    iface->cleanup_in_call_audio_channel_finish = mm_shared_simtech_voice_cleanup_in_call_audio_channel_finish;
+
+}
+
+static MMIfaceModemVoice *
+peek_parent_voice_interface (MMSharedSimtech *self)
+{
+    return iface_modem_voice_parent;
+}
+
+static void
+shared_simtech_init (MMSharedSimtech *iface)
+{
+    iface->peek_parent_location_interface = peek_parent_location_interface;
+    iface->peek_parent_voice_interface    = peek_parent_voice_interface;
 }
 
 static void
 mm_broadband_modem_simtech_class_init (MMBroadbandModemSimtechClass *klass)
 {
+    GObjectClass *object_class = G_OBJECT_CLASS (klass);
     MMBroadbandModemClass *broadband_modem_class = MM_BROADBAND_MODEM_CLASS (klass);
 
+    g_type_class_add_private (object_class, sizeof (MMBroadbandModemSimtechPrivate));
+
+    object_class->finalize = finalize;
+
     broadband_modem_class->setup_ports = setup_ports;
 }
