@@ -82,6 +82,9 @@ struct _MMBaseBearerPrivate {
     gchar *path;
     /* Status of this bearer */
     MMBearerStatus status;
+    /* Whether we must ignore all disconnection updates if they're
+     * detected by ModemManager itself. */
+    gboolean ignore_disconnection_reports;
     /* Configuration of the bearer */
     MMBearerProperties *config;
     /* Default IP family of this bearer */
@@ -192,20 +195,22 @@ static gboolean
 connection_monitor_cb (MMBaseBearer *self)
 {
     /* If the implementation knows how to load connection status, run it */
-    MM_BASE_BEARER_GET_CLASS (self)->load_connection_status (
-        self,
-        (GAsyncReadyCallback)load_connection_status_ready,
-        NULL);
+    if (self->priv->status == MM_BEARER_STATUS_CONNECTED)
+        MM_BASE_BEARER_GET_CLASS (self)->load_connection_status (
+            self,
+            (GAsyncReadyCallback)load_connection_status_ready,
+            NULL);
     return G_SOURCE_CONTINUE;
 }
 
 static gboolean
 initial_connection_monitor_cb (MMBaseBearer *self)
 {
-    MM_BASE_BEARER_GET_CLASS (self)->load_connection_status (
-        self,
-        (GAsyncReadyCallback)load_connection_status_ready,
-        NULL);
+    if (self->priv->status == MM_BEARER_STATUS_CONNECTED)
+        MM_BASE_BEARER_GET_CLASS (self)->load_connection_status (
+            self,
+            (GAsyncReadyCallback)load_connection_status_ready,
+            NULL);
 
     /* Add new monitor timeout at a higher rate */
     self->priv->connection_monitor_id = g_timeout_add_seconds (BEARER_CONNECTION_MONITOR_TIMEOUT,
@@ -302,6 +307,10 @@ reload_stats_ready (MMBaseBearer *self,
 static gboolean
 stats_update_cb (MMBaseBearer *self)
 {
+    /* Ignore stats update if we're not connected */
+    if (self->priv->status != MM_BEARER_STATUS_CONNECTED)
+        return G_SOURCE_CONTINUE;
+
     /* If the implementation knows how to update stat values, run it */
     if (!self->priv->reload_stats_unsupported &&
         MM_BASE_BEARER_GET_CLASS (self)->reload_stats &&
@@ -373,6 +382,8 @@ bearer_update_status (MMBaseBearer *self,
      * interface when going into disconnected state. */
     if (self->priv->status == MM_BEARER_STATUS_DISCONNECTED) {
         bearer_reset_interface_status (self);
+        /* Cleanup flag to ignore disconnection reports */
+        self->priv->ignore_disconnection_reports = FALSE;
         /* Stop statistics */
         bearer_stats_stop (self);
         /* Stop connection monitoring */
@@ -396,15 +407,25 @@ bearer_update_status_connected (MMBaseBearer *self,
         MM_GDBUS_BEARER (self),
         mm_bearer_ip_config_get_dictionary (ipv6_config));
 
+    /* If PPP is involved in the requested IP config, we must ignore
+     * all disconnection reports found via CGACT? polling or CGEV URCs.
+     * In this case, upper layers should always explicitly disconnect
+     * the bearer when ownership of the TTY is given back to MM. */
+    if ((ipv4_config && mm_bearer_ip_config_get_method (ipv4_config) == MM_BEARER_IP_METHOD_PPP) ||
+        (ipv6_config && mm_bearer_ip_config_get_method (ipv6_config) == MM_BEARER_IP_METHOD_PPP)) {
+        mm_dbg ("PPP is required for connection, will ignore disconnection reports");
+        self->priv->ignore_disconnection_reports = TRUE;
+    }
+
+    /* Update the property value */
+    self->priv->status = MM_BEARER_STATUS_CONNECTED;
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_STATUS]);
+
     /* Start statistics */
     bearer_stats_start (self);
 
     /* Start connection monitor, if supported */
     connection_monitor_start (self);
-
-    /* Update the property value */
-    self->priv->status = MM_BEARER_STATUS_CONNECTED;
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_STATUS]);
 }
 
 /*****************************************************************************/
@@ -709,9 +730,7 @@ connect_ready (MMBaseBearer *self,
         mm_dbg ("Couldn't connect bearer '%s': '%s'",
                 self->priv->path,
                 error->message);
-        if (g_error_matches (error,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_CANCELLED)) {
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
             /* Will launch disconnection */
             launch_disconnect = TRUE;
         } else
@@ -721,10 +740,8 @@ connect_ready (MMBaseBearer *self,
     else if (g_cancellable_is_cancelled (self->priv->connect_cancellable)) {
         mm_dbg ("Connected bearer '%s', but need to disconnect", self->priv->path);
         mm_bearer_connect_result_unref (result);
-        error = g_error_new (
-            MM_CORE_ERROR,
-            MM_CORE_ERROR_CANCELLED,
-            "Bearer got connected, but had to disconnect after cancellation request");
+        error = g_error_new (G_IO_ERROR, G_IO_ERROR_CANCELLED,
+                             "Bearer got connected, but had to disconnect after cancellation request");
         launch_disconnect = TRUE;
     }
     else {
@@ -916,6 +933,8 @@ handle_connect (MMBaseBearer *self,
                   MM_BASE_BEARER_MODEM, &ctx->modem,
                   NULL);
 
+    mm_dbg ("User request to connect bearer '%s'", self->priv->path);
+
     mm_base_modem_authorize (ctx->modem,
                              invocation,
                              MM_AUTHORIZATION_DEVICE_CONTROL,
@@ -943,7 +962,7 @@ disconnect_ready (MMBaseBearer *self,
     GError *error = NULL;
 
     if (!MM_BASE_BEARER_GET_CLASS (self)->disconnect_finish (self, res, &error)) {
-        mm_dbg ("Couldn't disconnect bearer '%s'", self->priv->path);
+        mm_dbg ("Couldn't disconnect bearer '%s': %s", self->priv->path, error->message);
         bearer_update_status (self, MM_BEARER_STATUS_CONNECTED);
         g_task_return_error (task, error);
     }
@@ -1107,6 +1126,8 @@ handle_disconnect (MMBaseBearer *self,
                   MM_BASE_BEARER_MODEM, &ctx->modem,
                   NULL);
 
+    mm_dbg ("User request to disconnect bearer '%s'", self->priv->path);
+
     mm_base_modem_authorize (ctx->modem,
                              invocation,
                              MM_AUTHORIZATION_DEVICE_CONTROL,
@@ -1256,13 +1277,46 @@ report_connection_status (MMBaseBearer *self,
         bearer_update_status (self, MM_BEARER_STATUS_DISCONNECTED);
 }
 
+/*
+ * This method is used exclusively in two different scenarios:
+ *  a) to report disconnections detected by ModemManager itself (e.g. based on
+ *     CGACT polling or CGEV URCs), applicable to bearers using both NET and
+ *     PPP data ports.
+ *  b) to report failed or successful connection attempts by plugins using NET
+ *     data ports that rely on vendor-specific URCs (e.g. Icera, MBM, Option
+ *     HSO).
+ *
+ * The method is also subclass-able because plugins may require specific
+ * cleanup operations to be done when a bearer is reported as disconnected.
+ * (e.g. the QMI or MBIM implementations require removing signal handlers).
+ *
+ * For all the scenarios involving a) the plugins are required to call the
+ * parent report_connection_status() implementation to report the
+ * DISCONNECTED state. For scenarios involving b) the parent reporting is not
+ * expected at all. In other words, the parent report_connection_status()
+ * is exclusively used in processing disconnections detected by ModemManager
+ * itself.
+ *
+ * If the bearer has been connected and it has required PPP method, we will
+ * ignore all disconnection reports because we cannot disconnect a PPP-based
+ * bearer before the upper layers have stopped using the TTY. In this case,
+ * we must wait for upper layers to detect the disconnection themselves (e.g.
+ * pppd should detect it) and disconnect the bearer through DBus.
+ */
 void
-mm_base_bearer_report_connection_status (MMBaseBearer *self,
-                                         MMBearerConnectionStatus status)
+mm_base_bearer_report_connection_status (MMBaseBearer             *self,
+                                         MMBearerConnectionStatus  status)
 {
+    if ((status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED) && self->priv->ignore_disconnection_reports) {
+        mm_dbg ("ignoring disconnection report for bearer '%s'", self->priv->path);
+        return;
+    }
+
     return MM_BASE_BEARER_GET_CLASS (self)->report_connection_status (self, status);
 }
 
+/*****************************************************************************/
+
 static void
 set_property (GObject *object,
               guint prop_id,
