@@ -59,7 +59,7 @@ typedef struct _MMBaseBearerPrivate MMBaseBearerPrivate;
 #define MM_BASE_BEARER_MODEM             "bearer-modem"
 #define MM_BASE_BEARER_STATUS            "bearer-status"
 #define MM_BASE_BEARER_CONFIG            "bearer-config"
-#define MM_BASE_BEARER_DEFAULT_IP_FAMILY "bearer-deafult-ip-family"
+#define MM_BASE_BEARER_DEFAULT_IP_FAMILY "bearer-default-ip-family"
 
 typedef enum { /*< underscore_name=mm_bearer_status >*/
     MM_BEARER_STATUS_DISCONNECTED,
@@ -101,6 +101,17 @@ struct _MMBaseBearerClass {
                                     GAsyncResult *res,
                                     GError **error);
 
+    /* Monitor connection status:
+     * NOTE: only CONNECTED or DISCONNECTED should be reported here; this method
+     * is used to poll for connection status once the connection has been
+     * established */
+    void (* load_connection_status) (MMBaseBearer *bearer,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data);
+    MMBearerConnectionStatus (* load_connection_status_finish) (MMBaseBearer *bearer,
+                                                                GAsyncResult *res,
+                                                                GError **error);
+
     /* Reload statistics */
     void (* reload_stats) (MMBaseBearer *bearer,
                            GAsyncReadyCallback callback,
