@@ -115,60 +115,71 @@ get_registration_state_context (MMIfaceModem3gpp *self)
     return ctx;
 }
 
-static gboolean
-reg_state_is_registered (MMModem3gppRegistrationState state)
-{
-    return state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME_SMS_ONLY ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING_SMS_ONLY ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME_CSFB_NOT_PREFERRED ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING_CSFB_NOT_PREFERRED;
-}
+#define REG_STATE_IS_REGISTERED(state)                                    \
+    (state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||                    \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING ||                 \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME_SMS_ONLY ||           \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING_SMS_ONLY ||        \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME_CSFB_NOT_PREFERRED || \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING_CSFB_NOT_PREFERRED)
+
+#define REG_STATE_IS_UNKNOWN_IDLE_DENIED(state)                           \
+    (state == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN ||                 \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_IDLE ||                    \
+     state == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED)
 
 static MMModem3gppRegistrationState
 get_consolidated_reg_state (RegistrationStateContext *ctx)
 {
+    MMModem3gppRegistrationState consolidated = MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
+
     /* Some devices (Blackberries for example) will respond to +CGREG, but
      * return ERROR for +CREG, probably because their firmware is just stupid.
      * So here we prefer the +CREG response, but if we never got a successful
      * +CREG response, we'll take +CGREG instead.
      */
     if (ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
-        ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING)
-        return ctx->cs;
+        ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING) {
+        consolidated = ctx->cs;
+        goto out;
+    }
     if (ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
-        ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING)
-        return ctx->ps;
+        ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING) {
+        consolidated = ctx->ps;
+        goto out;
+    }
     if (ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING)
-        return ctx->eps;
+        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING) {
+        consolidated = ctx->eps;
+        goto out;
+    }
 
     /* Searching? */
     if (ctx->cs  == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING ||
         ctx->ps  == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING ||
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING)
-         return MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING;
+        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING) {
+         consolidated = MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING;
+         goto out;
+    }
 
-    /* If one state is DENIED and the others are UNKNOWN, use DENIED */
-    if (ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED &&
-        ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN &&
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN)
-        return ctx->cs;
-    if (ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN &&
-        ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED &&
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN)
-        return ctx->ps;
-    if (ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN &&
-        ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN &&
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED)
-        return ctx->eps;
+    /* If at least one state is DENIED and the others are UNKNOWN or IDLE, use DENIED */
+    if ((ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED ||
+         ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED ||
+         ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_DENIED) &&
+        REG_STATE_IS_UNKNOWN_IDLE_DENIED (ctx->cs) &&
+        REG_STATE_IS_UNKNOWN_IDLE_DENIED (ctx->ps) &&
+        REG_STATE_IS_UNKNOWN_IDLE_DENIED (ctx->eps)) {
+        consolidated = MM_MODEM_3GPP_REGISTRATION_STATE_DENIED;
+        goto out;
+    }
 
     /* Emergency services? */
     if (ctx->cs  == MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY ||
         ctx->ps  == MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY ||
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY)
-         return MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY;
+        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY) {
+         consolidated = MM_MODEM_3GPP_REGISTRATION_STATE_EMERGENCY_ONLY;
+         goto out;
+    }
 
     /* Support for additional registration states reported when on LTE.
      *
@@ -188,17 +199,26 @@ get_consolidated_reg_state (RegistrationStateContext *ctx)
         ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING_CSFB_NOT_PREFERRED) {
         mm_warn ("3GPP CSFB registration state is consolidated: %s",
                  mm_modem_3gpp_registration_state_get_string (ctx->cs));
-        return ctx->cs;
+        consolidated = ctx->cs;
+        goto out;
     }
 
     /* Idle? */
     if (ctx->cs  == MM_MODEM_3GPP_REGISTRATION_STATE_IDLE ||
         ctx->ps  == MM_MODEM_3GPP_REGISTRATION_STATE_IDLE ||
-        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_IDLE)
-         return MM_MODEM_3GPP_REGISTRATION_STATE_IDLE;
+        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_IDLE) {
+         consolidated = MM_MODEM_3GPP_REGISTRATION_STATE_IDLE;
+         goto out;
+    }
 
-    /* Just unknown at this point */
-    return MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
+ out:
+    mm_dbg ("building consolidated registration state: cs '%s', ps '%s', eps '%s' --> '%s'",
+            mm_modem_3gpp_registration_state_get_string (ctx->cs),
+            mm_modem_3gpp_registration_state_get_string (ctx->ps),
+            mm_modem_3gpp_registration_state_get_string (ctx->eps),
+            mm_modem_3gpp_registration_state_get_string (consolidated));
+
+    return consolidated;
 }
 
 /*****************************************************************************/
@@ -314,7 +334,7 @@ run_registration_checks_ready (MMIfaceModem3gpp *self,
     }
 
     /* If we got registered, end registration checks */
-    if (reg_state_is_registered (current_registration_state)) {
+    if (REG_STATE_IS_REGISTERED (current_registration_state)) {
         /* Request immediate access tech and signal update: we may have changed
          * from home to roaming or viceversa, both registered states, so there
          * wouldn't be an explicit refresh triggered from the modem interface as
@@ -442,7 +462,7 @@ mm_iface_modem_3gpp_register_in_network (MMIfaceModem3gpp *self,
 
         /* If the modem is already registered and the last time it was asked
          * automatic registration, we're done */
-        if ((current_operator_code || reg_state_is_registered (reg_state)) &&
+        if ((current_operator_code || REG_STATE_IS_REGISTERED (reg_state)) &&
             !registration_state_context->manual_registration) {
             mm_dbg ("Already registered in network '%s',"
                     " automatic registration not launched...",
@@ -1303,7 +1323,7 @@ mm_iface_modem_3gpp_update_access_technologies (MMIfaceModem3gpp *self,
 
     /* Even if registration state didn't change, report access technology,
      * but only if something valid to report */
-    if (reg_state_is_registered (state) || ctx->reloading_registration_info) {
+    if (REG_STATE_IS_REGISTERED (state) || ctx->reloading_registration_info) {
         if (access_tech != MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN)
             mm_iface_modem_update_access_technologies (MM_IFACE_MODEM (self),
                                                        access_tech,
@@ -1337,7 +1357,7 @@ mm_iface_modem_3gpp_update_location (MMIfaceModem3gpp *self,
      * location updates, but only if something valid to report. For the case
      * where we're registering (loading current registration info after a state
      * change to registered), we also allow LAC/CID updates. */
-    if (reg_state_is_registered (state) || ctx->reloading_registration_info) {
+    if (REG_STATE_IS_REGISTERED (state) || ctx->reloading_registration_info) {
         if ((location_area_code > 0 || tracking_area_code > 0) && cell_id > 0)
             mm_iface_modem_location_3gpp_update_lac_tac_ci (MM_IFACE_MODEM_LOCATION (self),
                                                             location_area_code,
@@ -1420,7 +1440,7 @@ update_registration_state (MMIfaceModem3gpp *self,
     if (new_state == old_state)
         return;
 
-    if (reg_state_is_registered (new_state)) {
+    if (REG_STATE_IS_REGISTERED (new_state)) {
         MMModemState modem_state;
 
         /* If already reloading registration info, skip it */
