@@ -21,6 +21,7 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
+#include "mm-base-bearer.h"
 #include "mm-port-serial-at.h"
 
 #define MM_TYPE_IFACE_MODEM_3GPP               (mm_iface_modem_3gpp_get_type ())
@@ -34,6 +35,7 @@
 #define MM_IFACE_MODEM_3GPP_PS_NETWORK_SUPPORTED    "iface-modem-3gpp-ps-network-supported"
 #define MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED   "iface-modem-3gpp-eps-network-supported"
 #define MM_IFACE_MODEM_3GPP_IGNORED_FACILITY_LOCKS  "iface-modem-3gpp-ignored-facility-locks"
+#define MM_IFACE_MODEM_3GPP_INITIAL_EPS_BEARER      "iface-modem-3gpp-initial-eps-bearer"
 
 #define MM_IFACE_MODEM_3GPP_ALL_ACCESS_TECHNOLOGIES_MASK    \
     (MM_MODEM_ACCESS_TECHNOLOGY_GSM |                       \
@@ -145,6 +147,26 @@ struct _MMIfaceModem3gpp {
                                                                 GAsyncResult *res,
                                                                 GError **error);
 
+    /* Asynchronous initial default EPS bearer loading */
+    void                 (*load_initial_eps_bearer)        (MMIfaceModem3gpp     *self,
+                                                            GAsyncReadyCallback   callback,
+                                                            gpointer              user_data);
+    MMBearerProperties * (*load_initial_eps_bearer_finish) (MMIfaceModem3gpp     *self,
+                                                            GAsyncResult         *res,
+                                                            GError              **error);
+
+    /* Asynchronous initial default EPS bearer settings loading */
+    void                 (*load_initial_eps_bearer_settings)        (MMIfaceModem3gpp     *self,
+                                                                     GAsyncReadyCallback   callback,
+                                                                     gpointer              user_data);
+    MMBearerProperties * (*load_initial_eps_bearer_settings_finish) (MMIfaceModem3gpp     *self,
+                                                                     GAsyncResult         *res,
+                                                                     GError              **error);
+
+    /* Create initial default EPS bearer object */
+    MMBaseBearer * (*create_initial_eps_bearer) (MMIfaceModem3gpp   *self,
+                                                 MMBearerProperties *properties);
+
     /* Run CS/PS/EPS registration state checks..
      * Note that no registration state is returned, implementations should call
      * mm_iface_modem_3gpp_update_registration_state(). */
@@ -184,14 +206,6 @@ struct _MMIfaceModem3gpp {
                                           GAsyncResult *res,
                                           GError **error);
 
-    /* Loading of the subscription state property */
-    void (*load_subscription_state) (MMIfaceModem3gpp *self,
-                                     GAsyncReadyCallback callback,
-                                     gpointer user_data);
-    MMModem3gppSubscriptionState (*load_subscription_state_finish) (MMIfaceModem3gpp *self,
-                                                                    GAsyncResult *res,
-                                                                    GError **error);
-
     /* Scan current networks, expect a GList of MMModem3gppNetworkInfo */
     void (* scan_networks) (MMIfaceModem3gpp *self,
                             GAsyncReadyCallback callback,
@@ -208,6 +222,15 @@ struct _MMIfaceModem3gpp {
     gboolean (* set_eps_ue_mode_operation_finish) (MMIfaceModem3gpp               *self,
                                                    GAsyncResult                   *res,
                                                    GError                        **error);
+
+    /* Set initial EPS bearer settings */
+    void     (* set_initial_eps_bearer_settings)        (MMIfaceModem3gpp     *self,
+                                                         MMBearerProperties   *properties,
+                                                         GAsyncReadyCallback   callback,
+                                                         gpointer              user_data);
+    gboolean (* set_initial_eps_bearer_settings_finish) (MMIfaceModem3gpp     *self,
+                                                         GAsyncResult         *res,
+                                                         GError              **error);
 };
 
 GType mm_iface_modem_3gpp_get_type (void);
@@ -257,7 +280,12 @@ void mm_iface_modem_3gpp_update_access_technologies (MMIfaceModem3gpp *self,
                                                      MMModemAccessTechnology access_tech);
 void mm_iface_modem_3gpp_update_location            (MMIfaceModem3gpp *self,
                                                      gulong location_area_code,
+                                                     gulong tracking_area_code,
                                                      gulong cell_id);
+void mm_iface_modem_3gpp_update_pco_list            (MMIfaceModem3gpp *self,
+                                                     const GList *pco_list);
+void mm_iface_modem_3gpp_update_initial_eps_bearer  (MMIfaceModem3gpp *self,
+                                                     MMBearerProperties *properties);
 
 /* Run all registration checks */
 void mm_iface_modem_3gpp_run_registration_checks (MMIfaceModem3gpp *self,
