@@ -38,7 +38,9 @@
 #define MM_IFACE_MODEM_BEARER_LIST             "iface-modem-bearer-list"
 #define MM_IFACE_MODEM_SIM_HOT_SWAP_SUPPORTED  "iface-modem-sim-hot-swap-supported"
 #define MM_IFACE_MODEM_SIM_HOT_SWAP_CONFIGURED "iface-modem-sim-hot-swap-configured"
-#define MM_IFACE_MODEM_PERIODIC_SIGNAL_CHECK_DISABLED "iface-modem-periodic-signal-check-disabled"
+#define MM_IFACE_MODEM_CARRIER_CONFIG_MAPPING  "iface-modem-carrier-config-mapping"
+#define MM_IFACE_MODEM_PERIODIC_SIGNAL_CHECK_DISABLED      "iface-modem-periodic-signal-check-disabled"
+#define MM_IFACE_MODEM_PERIODIC_ACCESS_TECH_CHECK_DISABLED "iface-modem-periodic-access-tech-check-disabled"
 
 typedef struct _MMIfaceModem MMIfaceModem;
 
@@ -119,6 +121,7 @@ struct _MMIfaceModem {
 
     /* Loading of the UnlockRequired property */
     void (*load_unlock_required) (MMIfaceModem *self,
+                                  gboolean last_attempt,
                                   GAsyncReadyCallback callback,
                                   gpointer user_data);
     MMModemLock (*load_unlock_required_finish) (MMIfaceModem *self,
@@ -356,6 +359,26 @@ struct _MMIfaceModem {
     gboolean (*setup_sim_hot_swap_finish) (MMIfaceModem *self,
                                             GAsyncResult *res,
                                             GError **error);
+
+    /* Load carrier config */
+    void     (* load_carrier_config)        (MMIfaceModem         *self,
+                                             GAsyncReadyCallback   callback,
+                                             gpointer              user_data);
+    gboolean (* load_carrier_config_finish) (MMIfaceModem         *self,
+                                             GAsyncResult         *res,
+                                             gchar               **carrier_config_name,
+                                             gchar               **carrier_config_revision,
+                                             GError              **error);
+
+    /* Setup carrier config based on IMSI */
+    void     (* setup_carrier_config)        (MMIfaceModem         *self,
+                                              const gchar          *imsi,
+                                              const gchar          *carrier_config_mapping,
+                                              GAsyncReadyCallback   callback,
+                                              gpointer              user_data);
+    gboolean (* setup_carrier_config_finish) (MMIfaceModem         *self,
+                                              GAsyncResult         *res,
+                                              GError              **error);
 };
 
 GType mm_iface_modem_get_type (void);
@@ -381,8 +404,11 @@ gboolean mm_iface_modem_is_4g      (MMIfaceModem *self);
 gboolean mm_iface_modem_is_4g_only (MMIfaceModem *self);
 
 /* Helpers to query properties */
-const gchar *mm_iface_modem_get_model    (MMIfaceModem *self);
-const gchar *mm_iface_modem_get_revision (MMIfaceModem *self);
+const gchar *mm_iface_modem_get_model          (MMIfaceModem  *self);
+const gchar *mm_iface_modem_get_revision       (MMIfaceModem  *self);
+gboolean     mm_iface_modem_get_carrier_config (MMIfaceModem  *self,
+                                                const gchar  **name,
+                                                const gchar  **revision);
 
 /* Initialize Modem interface (async) */
 void     mm_iface_modem_initialize        (MMIfaceModem *self,
@@ -434,7 +460,8 @@ MMModemLock mm_iface_modem_update_lock_info_finish (MMIfaceModem *self,
                                                     GAsyncResult *res,
                                                     GError **error);
 
-MMUnlockRetries *mm_iface_modem_get_unlock_retries (MMIfaceModem *self);
+MMModemLock      mm_iface_modem_get_unlock_required (MMIfaceModem *self);
+MMUnlockRetries *mm_iface_modem_get_unlock_retries  (MMIfaceModem *self);
 
 void mm_iface_modem_update_unlock_retries (MMIfaceModem *self,
                                            MMUnlockRetries *unlock_retries);
