@@ -12,8 +12,8 @@
  *
  * Copyright (C) 2008 - 2009 Novell, Inc.
  * Copyright (C) 2009 - 2012 Red Hat, Inc.
- * Copyright (C) 2011 - 2012 Aleksander Morgado <aleksander@gnu.org>
  * Copyright (C) 2012 Google, Inc.
+ * Copyright (C) 2011 - 2019 Aleksander Morgado <aleksander@gnu.org>
  */
 
 #include <string.h>
@@ -27,8 +27,12 @@
 
 #include "mm-plugin-manager.h"
 #include "mm-plugin.h"
+#include "mm-shared.h"
 #include "mm-log.h"
 
+#define SHARED_PREFIX "libmm-shared"
+#define PLUGIN_PREFIX "libmm-plugin"
+
 static void initable_iface_init (GInitableIface *iface);
 
 G_DEFINE_TYPE_EXTENDED (MMPluginManager, mm_plugin_manager, G_TYPE_OBJECT, 0,
@@ -677,6 +681,10 @@ port_context_new (MMPluginManager *self,
  * (needs to be > MIN_WAIT_TIME_MSECS!!) */
 #define MIN_PROBING_TIME_MSECS 2500
 
+/* Additional time to wait for other ports to appear after the last port is
+ * exposed in the system. */
+#define EXTRA_PROBING_TIME_MSECS 1500
+
 /* The wait time we define must always be less than the probing time */
 G_STATIC_ASSERT (MIN_WAIT_TIME_MSECS < MIN_PROBING_TIME_MSECS);
 
@@ -716,11 +724,18 @@ struct _DeviceContext {
     /* Port support check contexts waiting to be run after min wait time */
     GList *wait_port_contexts;
 
-    /* Minimum probing_time. The device support check task cannot be finished
-     * before this timeout expires. Once the timeout is expired, the id is reset
-     * to 0. */
+    /* Minimum probing time, which is a timeout initialized as soon as the first
+     * port is added to the device context. The device support check task cannot
+     * be finished before this timeout expires. Once the timeout is expired, the
+     * id is reset to 0. */
     guint min_probing_time_id;
 
+    /* Extra probing time, which is a timeout refreshed every time a new port
+     * is added to the device context. The device support check task cannot be
+     * finished before this timeout expires. Once the timeout is expired, the id
+     * is reset to 0. */
+    guint extra_probing_time_id;
+
     /* Signal connection ids for the grabbed/released signals from the device.
      * These are the signals that will give us notifications of what ports are
      * available (or suddenly unavailable) in the device. */
@@ -741,6 +756,7 @@ device_context_unref (DeviceContext *device_context)
         g_assert (!device_context->released_id);
         g_assert (!device_context->min_wait_time_id);
         g_assert (!device_context->min_probing_time_id);
+        g_assert (!device_context->extra_probing_time_id);
         g_assert (!device_context->port_contexts);
 
         /* The device support check task must have been completed previously */
@@ -812,6 +828,23 @@ device_context_complete (DeviceContext *device_context)
 {
     GTask *task;
 
+    /* If the context is completed before the 2500ms minimum probing time, we need to wait
+     * until that happens, so that we give enough time to udev/hotplug to report the
+     * new port additions. */
+    if (device_context->min_probing_time_id) {
+        mm_dbg ("[plugin manager] task %s: all port probings completed, but not reached min probing time yet",
+                device_context->name);
+        return;
+    }
+
+    /* If the context is completed less than 1500ms before the last port was exposed,
+     * wait some more. */
+    if (device_context->extra_probing_time_id) {
+        mm_dbg ("[plugin manager] task %s: all port probings completed, but not reached extra probing time yet",
+                device_context->name);
+        return;
+    }
+
     /* Steal the task from the context */
     g_assert (device_context->task);
     task = device_context->task;
@@ -831,15 +864,8 @@ device_context_complete (DeviceContext *device_context)
         device_context->released_id = 0;
     }
 
-    /* Remove timeouts, if still around */
-    if (device_context->min_wait_time_id) {
-        g_source_remove (device_context->min_wait_time_id);
-        device_context->min_wait_time_id = 0;
-    }
-    if (device_context->min_probing_time_id) {
-        g_source_remove (device_context->min_probing_time_id);
-        device_context->min_probing_time_id = 0;
-    }
+    /* On completion, the minimum wait time must have been already elapsed */
+    g_assert (!device_context->min_wait_time_id);
 
     /* Task completion */
     if (!device_context->best_plugin)
@@ -1071,6 +1097,18 @@ device_context_min_probing_time_elapsed (DeviceContext *device_context)
     return G_SOURCE_REMOVE;
 }
 
+static gboolean
+device_context_extra_probing_time_elapsed (DeviceContext *device_context)
+{
+    device_context->extra_probing_time_id = 0;
+
+    mm_dbg ("[plugin manager] task %s: extra probing time elapsed", device_context->name);
+
+    /* Wakeup the device context logic */
+    device_context_continue (device_context);
+    return G_SOURCE_REMOVE;
+}
+
 static void
 device_context_run_port_context (DeviceContext *device_context,
                                  PortContext   *port_context)
@@ -1202,6 +1240,13 @@ device_context_port_grabbed (DeviceContext  *device_context,
         return;
     }
 
+    /* Refresh the extra probing timeout. */
+    if (device_context->extra_probing_time_id)
+        g_source_remove (device_context->extra_probing_time_id);
+    device_context->extra_probing_time_id = g_timeout_add (EXTRA_PROBING_TIME_MSECS,
+                                                           (GSourceFunc) device_context_extra_probing_time_elapsed,
+                                                           device_context);
+
     /* Setup a new port context for the newly grabbed port */
     port_context = port_context_new (self,
                                      device_context->name,
@@ -1256,6 +1301,20 @@ device_context_cancel (DeviceContext *device_context)
         g_list_foreach (device_context->port_contexts, (GFunc) port_context_cancel, NULL);
     }
 
+    /* Cancel all timeouts */
+    if (device_context->min_wait_time_id) {
+        g_source_remove (device_context->min_wait_time_id);
+        device_context->min_wait_time_id = 0;
+    }
+    if (device_context->min_probing_time_id) {
+        g_source_remove (device_context->min_probing_time_id);
+        device_context->min_probing_time_id = 0;
+    }
+    if (device_context->extra_probing_time_id) {
+        g_source_remove (device_context->extra_probing_time_id);
+        device_context->extra_probing_time_id = 0;
+    }
+
     /* Wakeup the device context logic. If we were still waiting for the
      * min probing time, this will complete the device context. */
     device_context_continue (device_context);
@@ -1273,6 +1332,7 @@ device_context_run (MMPluginManager     *self,
     g_assert (!device_context->released_id);
     g_assert (!device_context->min_wait_time_id);
     g_assert (!device_context->min_probing_time_id);
+    g_assert (!device_context->extra_probing_time_id);
 
     /* Connect to device port grabbed/released notifications from the device */
     device_context->grabbed_id = g_signal_connect_swapped (device_context->device,
@@ -1496,6 +1556,36 @@ mm_plugin_manager_peek_plugin (MMPluginManager *self,
 
 /*****************************************************************************/
 
+static void
+register_plugin_whitelist_tags (MMPluginManager *self,
+                                MMPlugin        *plugin)
+{
+    const gchar **tags;
+    guint         i;
+
+    if (!mm_filter_check_rule_enabled (self->priv->filter, MM_FILTER_RULE_PLUGIN_WHITELIST))
+        return;
+
+    tags = mm_plugin_get_allowed_udev_tags (plugin);
+    for (i = 0; tags && tags[i]; i++)
+        mm_filter_register_plugin_whitelist_tag (self->priv->filter, tags[i]);
+}
+
+static void
+register_plugin_whitelist_product_ids (MMPluginManager *self,
+                                       MMPlugin        *plugin)
+{
+    const mm_uint16_pair *product_ids;
+    guint                 i;
+
+    if (!mm_filter_check_rule_enabled (self->priv->filter, MM_FILTER_RULE_PLUGIN_WHITELIST))
+        return;
+
+    product_ids = mm_plugin_get_allowed_product_ids (plugin);
+    for (i = 0; product_ids && product_ids[i].l; i++)
+        mm_filter_register_plugin_whitelist_product_id (self->priv->filter, product_ids[i].l, product_ids[i].r);
+}
+
 static MMPlugin *
 load_plugin (const gchar *path)
 {
@@ -1509,7 +1599,7 @@ load_plugin (const gchar *path)
     /* Get printable UTF-8 string of the path */
     path_display = g_filename_display_name (path);
 
-    module = g_module_open (path, G_MODULE_BIND_LAZY);
+    module = g_module_open (path, 0);
     if (!module) {
         mm_warn ("[plugin manager] could not load plugin '%s': %s", path_display, g_module_error ());
         goto out;
@@ -1543,9 +1633,10 @@ load_plugin (const gchar *path)
     }
 
     plugin = (*plugin_create_func) ();
-    if (plugin)
+    if (plugin) {
+        mm_dbg ("[plugin manager] loaded plugin '%s' from '%s'", mm_plugin_get_name (plugin), path_display);
         g_object_weak_ref (G_OBJECT (plugin), (GWeakNotify) g_module_close, module);
-    else
+    } else
         mm_warn ("[plugin manager] could not load plugin '%s': initialization failed", path_display);
 
 out:
@@ -1557,6 +1648,60 @@ out:
     return plugin;
 }
 
+static void
+load_shared (const gchar *path)
+{
+    GModule      *module;
+    gchar        *path_display;
+    const gchar **shared_name = NULL;
+    gint         *major_shared_version;
+    gint         *minor_shared_version;
+
+    /* Get printable UTF-8 string of the path */
+    path_display = g_filename_display_name (path);
+
+    module = g_module_open (path, 0);
+    if (!module) {
+        mm_warn ("[plugin manager] could not load shared '%s': %s", path_display, g_module_error ());
+        goto out;
+    }
+
+    if (!g_module_symbol (module, "mm_shared_major_version", (gpointer *) &major_shared_version)) {
+        mm_warn ("[plugin manager] could not load shared '%s': Missing major version info", path_display);
+        goto out;
+    }
+
+    if (*major_shared_version != MM_SHARED_MAJOR_VERSION) {
+        mm_warn ("[plugin manager] could not load shared '%s': Shared major version %d, %d is required",
+                 path_display, *major_shared_version, MM_SHARED_MAJOR_VERSION);
+        goto out;
+    }
+
+    if (!g_module_symbol (module, "mm_shared_minor_version", (gpointer *) &minor_shared_version)) {
+        mm_warn ("[plugin manager] could not load shared '%s': Missing minor version info", path_display);
+        goto out;
+    }
+
+    if (*minor_shared_version != MM_SHARED_MINOR_VERSION) {
+        mm_warn ("[plugin manager] could not load shared '%s': Shared minor version %d, %d is required",
+                   path_display, *minor_shared_version, MM_SHARED_MINOR_VERSION);
+        goto out;
+    }
+
+    if (!g_module_symbol (module, "mm_shared_name", (gpointer *) &shared_name)) {
+        mm_warn ("[plugin manager] could not load shared '%s': Missing name", path_display);
+        goto out;
+    }
+
+    mm_dbg ("[plugin manager] loaded shared '%s' utils from '%s'", *shared_name, path_display);
+
+out:
+    if (module && !(*shared_name))
+        g_module_close (module);
+
+    g_free (path_display);
+}
+
 static gboolean
 load_plugins (MMPluginManager *self,
               GError **error)
@@ -1564,6 +1709,9 @@ load_plugins (MMPluginManager *self,
     GDir *dir = NULL;
     const gchar *fname;
     gchar *plugindir_display = NULL;
+    GList *shared_paths = NULL;
+    GList *plugin_paths = NULL;
+    GList *l;
 
     if (!g_module_supported ()) {
         g_set_error (error,
@@ -1588,27 +1736,36 @@ load_plugins (MMPluginManager *self,
     }
 
     while ((fname = g_dir_read_name (dir)) != NULL) {
-        gchar *path;
-        MMPlugin *plugin;
-
         if (!g_str_has_suffix (fname, G_MODULE_SUFFIX))
             continue;
+        if (g_str_has_prefix (fname, SHARED_PREFIX))
+            shared_paths = g_list_prepend (shared_paths, g_module_build_path (self->priv->plugin_dir, fname));
+        else if (g_str_has_prefix (fname, PLUGIN_PREFIX))
+            plugin_paths = g_list_prepend (plugin_paths, g_module_build_path (self->priv->plugin_dir, fname));
+    }
 
-        path = g_module_build_path (self->priv->plugin_dir, fname);
-        plugin = load_plugin (path);
-        g_free (path);
+    /* Load all shared utils */
+    for (l = shared_paths; l; l = g_list_next (l))
+        load_shared ((const gchar *)(l->data));
 
+    /* Load all plugins */
+    for (l = plugin_paths; l; l = g_list_next (l)) {
+        MMPlugin *plugin;
+
+        plugin = load_plugin ((const gchar *)(l->data));
         if (!plugin)
             continue;
 
-        mm_dbg ("[plugin manager] loaded plugin '%s'", mm_plugin_get_name (plugin));
-
         if (g_str_equal (mm_plugin_get_name (plugin), MM_PLUGIN_GENERIC_NAME))
             /* Generic plugin */
             self->priv->generic = plugin;
         else
             /* Vendor specific plugin */
             self->priv->plugins = g_list_append (self->priv->plugins, plugin);
+
+        /* Register plugin whitelist rules in filter, if any */
+        register_plugin_whitelist_tags (self, plugin);
+        register_plugin_whitelist_product_ids (self, plugin);
     }
 
     /* Check the generic plugin once all looped */
@@ -1629,6 +1786,8 @@ load_plugins (MMPluginManager *self,
             g_list_length (self->priv->plugins) + !!self->priv->generic);
 
 out:
+    g_list_free_full (shared_paths, g_free);
+    g_list_free_full (plugin_paths, g_free);
     if (dir)
         g_dir_close (dir);
     g_free (plugindir_display);
