@@ -815,8 +815,7 @@ port_serial_response_wait_cancelled (GCancellable *cancellable,
     /* FIXME: This is not completely correct - if the response finally arrives and there's
      * some other command waiting for response right now, the other command will
      * get the output of the cancelled command. Not sure what to do here. */
-    error = g_error_new_literal (MM_CORE_ERROR,
-                                 MM_CORE_ERROR_CANCELLED,
+    error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED,
                                  "Waiting for the reply cancelled");
     /* Note: may complete last operation and unref the MMPortSerial */
     port_serial_got_response (self, NULL, error);
@@ -1387,7 +1386,10 @@ _close_internal (MMPortSerial *self, gboolean force)
         return;
 
     if (self->priv->connected_id) {
-        g_signal_handler_disconnect (self, self->priv->connected_id);
+        /* Don't assume it's always connected, because it may be automatically connected during
+         * object disposal, and this method is also called in finalize() */
+        if (g_signal_handler_is_connected (self, self->priv->connected_id))
+            g_signal_handler_disconnect (self, self->priv->connected_id);
         self->priv->connected_id = 0;
     }
 
@@ -1563,10 +1565,7 @@ port_serial_reopen_cancel (MMPortSerial *self)
     task = self->priv->reopen_task;
     self->priv->reopen_task = NULL;
 
-    g_task_return_new_error (task,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_CANCELLED,
-                             "Reopen cancelled");
+    g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Reopen cancelled");
     g_object_unref (task);
 }
 
@@ -1593,9 +1592,18 @@ reopen_do (MMPortSerial *self)
         }
     }
 
-    if (error)
+    if (error) {
+        /* An error during port reopening may mean that the device is
+         * already gone. Note that we won't get a HUP in the TTY when
+         * the port is gone during the reopen wait time, because there's
+         * no channel I/O monitoring in place.
+         *
+         * If we ever see this, we'll flag the port as forced close right
+         * away, because the open count would anyway be broken afterwards.
+         */
+        port_serial_close_force (self);
         g_task_return_error (task, error);
-    else
+    } else
         g_task_return_boolean (task, TRUE);
     g_object_unref (task);
 
@@ -1725,11 +1733,21 @@ mm_port_serial_flash_finish (MMPortSerial *port,
     return g_task_propagate_boolean (G_TASK (res), error);
 }
 
+static gboolean
+flash_cancel_cb (GTask *task)
+{
+    g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Flash cancelled");
+    g_object_unref (task);
+    return G_SOURCE_REMOVE;
+}
+
 void
 mm_port_serial_flash_cancel (MMPortSerial *self)
 {
-    GTask *task;
+    FlashContext *ctx;
+    GTask        *task;
 
+    /* Do nothing if there is no flash task */
     if (!self->priv->flash_task)
         return;
 
@@ -1737,11 +1755,18 @@ mm_port_serial_flash_cancel (MMPortSerial *self)
     task = self->priv->flash_task;
     self->priv->flash_task = NULL;
 
-    g_task_return_new_error (task,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_CANCELLED,
-                             "Flash cancelled");
-    g_object_unref (task);
+    /* If flash operation is scheduled, unschedule it */
+    ctx = g_task_get_task_data (task);
+    if (ctx->flash_id) {
+        g_source_remove (ctx->flash_id);
+        ctx->flash_id = 0;
+    }
+
+    /* Schedule task to be cancelled in an idle.
+     * We do NOT want this cancellation to happen right away,
+     * because the object reference in the flashing task may
+     * be the last one valid. */
+    g_idle_add ((GSourceFunc)flash_cancel_cb, task);
 }
 
 static gboolean
